OPTION_TYPE_CALL = 0
OPTION_TYPE_PUT = 1
OPTION_TYPE_UNDEFINED = -1


INSTRUMENT_TYPE_STOCK = 'stock'
INSTRUMENT_TYPE_FUTURE = 'future'
INSTRUMENT_TYPE_OPTION = 'option'
INSTRUMENT_TYPE_UNDEFINED = 'undefined'


INSTRUMENT_EXECUTION_BUY = 1
INSTRUMENT_EXECUTION_SELL = -1



import time
from backtester.dataSource.data_source import DataSource


class LogfileDataSource(DataSource):
    def __init__(self, fileName):
        self.fileName = fileName
        self.file = open(fileName, "r")
        self.file.seek(0, 2)
        self.unfinishedLine = ''

    '''
    Assume line in processLine is a complete line.
    override this method to return an instrument
    '''
    def processLineIntoInstrumentUpdate(self, line):
        return line

    def emitInstrumentUpdates(self):
        while True:
            readLine = self.file.readline()
            if readLine:
                self.unfinishedLine = self.unfinishedLine + readLine
                if self.unfinishedLine.endswith('\n'):
                    instrumentUpdate = self.processLine(self.unfinishedLine)
                    self.unfinishedLine = ''
                    if instrumentUpdate:
                        yield(instrumentUpdate)
            else:
                time.sleep(0.1)



from datetime import datetime, timedelta
from backtester.instrumentUpdates import *
from backtester.constants import *
from backtester.logger import *
from backtester.dataSource.data_source import DataSource
import os
import os.path
import pandas as pd
import csv
from bs4 import BeautifulSoup
import urllib2
from backtester.dataSource.data_source_utils import downloadFileFromYahoo, groupAndSortByTimeUpdates
from urllib import quote

TYPE_LINE_UNDEFINED = 0
TYPE_LINE_HEADER = 1
TYPE_LINE_DATA = 2


def checkDate(lineItem):
    try:
        datetime.strptime(lineItem, '%d-%b-%Y')  # 3-Aug-15
        return True
    except ValueError:
        try:
            datetime.strptime(lineItem, '%Y-%m-%d')  # 3-Aug-15
            return True
        except ValueError:
            return False


def checkTimestamp(lineItem):
    return True


def isFloat(string):
    try:
        return float(string) or float(string) == 0.0
    except ValueError:  # if string is not a number
        return False

# Returns the type of lineItems


def validateLineItem(lineItems, lineLength):
    if len(lineItems) == lineLength:
        if lineItems[0] == "Date":
            return TYPE_LINE_HEADER
        elif checkDate(lineItems[0]) and isFloat(lineItems[2]) and isFloat(lineItems[3]) and isFloat(lineItems[4]) and isFloat(lineItems[5]) and isFloat(lineItems[6]):
            return TYPE_LINE_DATA
            # Date,Prev Close,Open,High,Low,Last,Close,Average,Total Traded Quantity,Turnover,No. of Trades,Deliverable Qty,% Dly Qt to Traded Qty
    print(len(lineItems), lineLength, checkDate(lineItems[0]))
    logInfo('Bad Line')
    return TYPE_LINE_UNDEFINED


def parseDataLine(lineItems, lineLength):
    if (len(lineItems) != lineLength):
        logInfo('Bad Line')
        return None
    openPrice = float(lineItems[2])
    high = float(lineItems[3])
    low = float(lineItems[4])
    last = float(lineItems[5])
    closePrice = float(lineItems[6])
    average = float(lineItems[7])
    volume = float(lineItems[8])
    return {'open': openPrice,
            'high': high,
            'low': low,
            'last': last,
            'close': closePrice,
            'average': average,
            'volume': volume}


class InstrumentsFromFile():
    def __init__(self, fileName, instrumentId):
        self.fileName = fileName
        self.instrumentId = instrumentId
        self.currentInstrumentSymbol = instrumentId
        self.currentTimeOfUpdate = None
        self.currentBookData = None

    def processLine(self, line, lineLength):
        lineItems = line.split(',')
        lineItemType = validateLineItem(lineItems, lineLength)
        if (lineItemType == TYPE_LINE_DATA):
            inst = None
            if self.currentInstrumentSymbol is not None:
                try:
                    self.currentTimeOfUpdate = datetime.strptime(lineItems[0], '%d-%b-%Y')
                except ValueError:
                    self.currentTimeOfUpdate = datetime.strptime(lineItems[0], '%Y-%m-%d')
                self.currentInstrumentSymbol = self.instrumentId
                self.currentBookData = parseDataLine(lineItems, lineLength)
                if self.currentBookData is None:
                    return None
                # right now only works for stocks
                inst = StockInstrumentUpdate(stockInstrumentId=self.instrumentId,
                                             tradeSymbol=self.currentInstrumentSymbol,
                                             timeOfUpdate=self.currentTimeOfUpdate,
                                             bookData=self.currentBookData)
                return inst
        return None

    def processLinesIntoInstruments(self, lineLength):
        with open(self.fileName, "r") as ins:
            instruments = []
            for line in ins:
                inst = self.processLine(line, lineLength)
                if inst is not None:
                    instruments.append(inst)
            return instruments


class FutureNSEStockDataSource(DataSource):
    def __init__(self, cachedFolderName, instrumentIds, startDateStr, endDateStr):
        self.startDate = datetime.strptime(startDateStr, "%Y/%m/%d")
        self.endDate = datetime.strptime(endDateStr, "%Y/%m/%d")
        self.cachedFolderName = cachedFolderName
        self.instrumentIds = instrumentIds
        self.currentDate = self.startDate
        self.lineLength = 13

    def getResponseFromUrl(self, url, isxml):
        hdr = {'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11',
               'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
               'Accept-Language': 'en-US,en;q=0.8',
               'Host': 'nseindia.com',
               'Connection': 'keep-alive',
               'Referer': 'https://nseindia.com/products/content/equities/equities/eq_security.htm',
               'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.3'}
        if isxml:
            hdr['X-Requested-With'] = 'XMLHttpRequest'
        req = urllib2.Request(url, headers=hdr)
        try:
            page = urllib2.urlopen(req)
            content = page.read()
            return content
        except urllib2.HTTPError, e:
            print e.fp.read()
            return None

    def getInitialSymbolCountUrl(self, stock):
        url = 'https://nseindia.com/marketinfo/sym_map/symbolCount.jsp?symbol=' + stock
        return url

    def getSymbolCountForStock(self, stock):
        url = self.getInitialSymbolCountUrl(stock)
        return self.getResponseFromUrl(url, False).strip()

    def getDataUrl(self, stock, symbolCount, exp, start, end):
        parameters = {'instrumentType': 'FUTSTK',
                      'symbol': stock,
                      'expiryDate': exp,
                      'optionType': 'select',
                      'strikePrice': '',
                      'dateRange': '',
                      'fromDate': start,
                      'toDate': end,
                      'segmentLink': '9',
                      'symbolCount': ''
                      }
        orderedKeys = ['instrumentType', 'symbol', 'expiryDate', 'optionType', 'strikePrice', 'dateRange', 'fromDate', 'toDate', 'segmentLink', 'symbolCount']
        encodedparams = "&".join("%s=%s" % (key, quote(
            parameters[key], safe="+")) for key in orderedKeys)
        return 'https://www.nseindia.com/products/dynaContent/common/productsSymbolMapping.jsp?' + encodedparams

    def getDataResponseForStock(self, stock, symbolCount, exp, start, end):
        url = self.getDataUrl(stock, symbolCount, exp, start, end)
        return self.getResponseFromUrl(url, True)

    def parseHtmlToCSV(self, soup, outputCsvFile):
        data = soup.find(id="csvContentDiv")
        if data is None:
            print 'No data'
            return
        data = data.get_text()
        dataRows = data.split(":")
        headers = [x.replace('"', '').strip() for x in dataRows[0].split(",")[2:]]
        headers = [x.replace(' Price', '').strip() for x in headers]
        rows = []
        for row in dataRows[1:-1]:
            rows.append([x.replace('"', '').strip() for x in row.split(",")[2:]])

        if not os.path.isfile(outputCsvFile):
            f = open(outputCsvFile, 'wb')
            writer = csv.writer(f)
            writer.writerow(headers)
            f.close()

        with open(outputCsvFile, 'ab') as f:
            writer = csv.writer(f)
            writer.writerows(row for row in rows if row)
            f.close()

    def parseNSEUrl(self, stock, exp, start, end, outputCsvFile):
        symbolCount = self.getSymbolCountForStock(stock)
        content = self.getDataResponseForStock(stock, symbolCount, exp, start, end)
        soup = BeautifulSoup(content, 'lxml')
        self.parseHtmlToCSV(soup, outputCsvFile)

    def downloadFile(self, instrumentId, fileName):
        logInfo('Downloading %s' % fileName)
        dates = ['2017/03/30', '2017/04/27', '2017/05/25', '2017/06/29', '2017/07/27', '2017/08/31']

        for date in dates:
            exp = datetime.strptime(date, "%Y/%m/%d")
            start = exp + timedelta(days = -1)
            end = exp + timedelta(days = 1)
            self.parseNSEUrl(instrumentId, datetime.strftime(exp, '%d-%m-%Y'), datetime.strftime(start, '%d-%m-%Y'), datetime.strftime(end, '%d-%m-%Y'), fileName)

    def getFileName(self, instrumentType, instrumentId):
        return '%s/%s_%s_%s_%s.csv' % (self.cachedFolderName, instrumentId, instrumentType, self.startDate.strftime("%Y%m%d"), self.endDate.strftime("%Y%m%d"))

    def emitInstrumentUpdates(self, adjustPrice=True):
        allInstrumentUpdates = []

        for instrumentId in self.instrumentIds:
            fileName = self.getFileName(INSTRUMENT_TYPE_STOCK, instrumentId)

            if not os.path.exists(self.cachedFolderName):
                os.mkdir(self.cachedFolderName, 0755)

            if not os.path.isfile(fileName):
                self.downloadFile(instrumentId, fileName)
                if adjustPrice:
                    self.adjustPriceForSplitAndDiv(instrumentId, fileName)

            fileHandler = InstrumentsFromFile(fileName=fileName, instrumentId=instrumentId)
            instrumentUpdates = fileHandler.processLinesIntoInstruments(self.lineLength)
            allInstrumentUpdates = allInstrumentUpdates + instrumentUpdates
        groupedInstrumentUpdates = groupAndSortByTimeUpdates(allInstrumentUpdates)
        for timeOfUpdate, instrumentUpdates in groupedInstrumentUpdates:
            yield([timeOfUpdate, instrumentUpdates])

    def adjustPriceForSplitAndDiv(self, instrumentId, fileName):
        divFile = self.getFileName('div', instrumentId)
        splitFile = self.getFileName('split', instrumentId)
        if not (os.path.isfile(divFile) and os.path.isfile(splitFile)):
            downloadFileFromYahoo(self.startDate, self.endDate, '%s.NS' % instrumentId, divFile, event='div')
            downloadFileFromYahoo(self.startDate, self.endDate, '%s.NS' % instrumentId, splitFile, event='split')
        div = pd.read_csv(divFile, engine='python', index_col='Date', parse_dates=True)
        split = pd.read_csv(splitFile, engine='python', index_col='Date', parse_dates=True)
        prices = pd.read_csv(fileName, engine='python', index_col='Date', parse_dates=True)
        temp = pd.concat([div, prices], axis=1).fillna(0)
        interim = (temp['Close'] - temp['Dividends']) / temp['Close']
        multiplier1 = interim.sort_index(ascending=False).cumprod().sort_index(ascending=True)
        temp2 = split['Stock Splits'].str.split('/', expand=True)
        if len(temp2.index) > 0:
            temp_mult = pd.to_numeric(temp2[1]) / pd.to_numeric(temp2[0])
            multiplier2 = temp_mult.sort_index(ascending=False).cumprod().sort_index(ascending=True)
        else:
            multiplier2 = pd.Series(1, index=multiplier1.index)
        multiplier = pd.concat([multiplier1, multiplier2], axis=1).fillna(method='bfill').fillna(1)
        multiplier[1] = multiplier[1].shift(-1).fillna(1)
        temp['Close'] = temp['Close'] * multiplier[0] * multiplier[1]
        temp['Open'] = temp['Open'] * multiplier[0] * multiplier[1]
        temp['High'] = temp['High'] * multiplier[0] * multiplier[1]
        temp['Low'] = temp['Low'] * multiplier[0] * multiplier[1]
        temp['Last'] = temp['Last'] * multiplier[0] * multiplier[1]
        temp['Average'] = temp['Average'] * multiplier[0] * multiplier[1]
        temp['Total Traded Quantity'] = temp['Total Traded Quantity'] / multiplier[1]
        temp['Turnover'] = temp['Turnover'] * multiplier[0]
        temp['Deliverable Qty'] = temp['Deliverable Qty'] / multiplier[1]

        del temp['Dividends']
        temp.to_csv(fileName)


if __name__ == "__main__":
    stockList = ['JPASSOCIAT','GMRINFRA','ASHOKLEY','SUZLON','IBREALEST','RELCAPITAL','ADANIENT','DLF','IDFC','SOUTHBANK','VEDL','SAIL','IDFCBANK','FEDERALBNK','BANKBARODA','MANAPPURAM','HINDALCO',
    'TATASTEEL','SBIN','JINDALSTEL','RCOM','HDIL','L&TFH','TATAMOTORS','NMDC','ADANIPOWER','HCC','INDIACEM','ICICIBANK','SUNTV','RELINFRA','PNB','JSWSTEEL','APOLLOTYRE','MOTHERSUMI',
    'TATAGLOBAL','RECLTD','ONGC','RPOWER','IDEA','JSWENERGY','IFCI','DHFL','TV18BRDCST','CANBK','BANKINDIA','ORIENTBANK','PFC','EXIDEIND','NCC','KTKBANK','BHARATFIN','M&MFIN','GRASIM',
    'NATIONALUM','AXISBANK','UNIONBANK','AUROPHARMA','INFIBEAM','BEL','ESCORTS','FORTIS','ALBK','HINDPETRO','MRPL','IDBI','TATAPOWER','PTC','NHPC','NTPC','SUNPHARMA','IOC','BPCL','INFY',
    'RELIANCE','DISHTV','JUSTDIAL','ITC','BHEL','SYNDIBANK','TATAMTRDVR','HINDZINC','CESC','GAIL','JISLJALEQS','COALINDIA','PCJEWELLER','ANDHRABANK','CGPOWER','POWERGRID','GSFC','AMBUJACEM',
    'ADANIPORTS','SREINFRA','EQUITAS','IRB','DCBBANK','YESBANK','LICHSGFIN','VOLTAS','TVSMOTOR','RDEL','KOTAKBANK','SRTRANSFIN','UJJIVAN','WIPRO','PETRONET','BALRAMCHIN','BHARTIARTL',
    'HDFCBANK']
    for stock in stockList:
        ds = FutureNSEStockDataSource(cachedFolderName='nseData',
                                 instrumentIds=[stock],
                                 startDateStr='2007/01/01',
                                 endDateStr='2007/01/08')
        ds.downloadFile(stock, stock + '_role')



from backtester.constants import *
from backtester.lookback_data import LookbackData
from backtester.features.feature_config import FeatureConfig
from backtester.instruments import *
from backtester.logger import *
from itertools import chain


def getCompulsoryMarketFeatureConfigs(tsParams):
    countDict = {'featureKey': 'count',
                 'featureId': 'count'}
    profitlossConfigDict = {'featureKey': 'pnl',
                            'featureId': 'pnl',
                            'params': {'instrument_pnl_feature': 'pnl'}}
    capitalConfigDict = {'featureKey': 'capital',
                         'featureId': 'capital',
                         'params': {'initial_capital': tsParams.getStartingCapital(),
                                    'pnl': 'pnl'}}
    portfoliovalueConfigDict = {'featureKey': 'portfolio_value',
                                'featureId': 'portfolio_value',
                                'params': {'initial_capital': tsParams.getStartingCapital(),
                                           'pnl': 'pnl'}}
    varianceConfigDict = {'featureKey': 'variance',
                          'featureId': 'variance',
                          'params': {'pnlKey': 'pnl',
                                        'countKey': 'count'}}
    maxCapitalUsageConfigDict = {'featureKey': 'capitalUsage',
                                 'featureId': 'maxCapitalUsage',
                                 'params': {'initial_capital': tsParams.getStartingCapital(),
                                            'capitalKey': 'capital'}}
    maxDrawdownConfigDict = {'featureKey': 'maxDrawdown',
                             'featureId': 'maxDrawdown',
                             'params': {'portfolioValueKey': 'portfolio_value'}}
    profitlossRatioConfigDict = {'featureKey': 'pl_ratio',
                                 'featureId': 'pl_ratio',
                                 'params': {'pnlKey': 'pnl',
                                            'countKey': 'count'}}
    compulsoryConfigDicts = [countDict, profitlossConfigDict, capitalConfigDict, portfoliovalueConfigDict,
                             varianceConfigDict, maxCapitalUsageConfigDict, maxDrawdownConfigDict, profitlossRatioConfigDict]
    compulsoryMarketFeatureConfigs = list(map(lambda x: FeatureConfig(x), compulsoryConfigDicts))
    return compulsoryMarketFeatureConfigs


class InstrumentManager:
    def __init__(self, tsParams):
        self.tsParams = tsParams
        self.__instrumentsDict = {}
        # TODO: create a different place to hold different types of instruments
        featureConfigs = tsParams.getMarketFeatureConfigs()
        self.__compulsoryFeatureConfigs = getCompulsoryMarketFeatureConfigs(tsParams)
        columns = map(lambda x: x.getFeatureKey(), featureConfigs)
        compulsoryColumns = map(lambda x: x.getFeatureKey(), self.__compulsoryFeatureConfigs)
        self.__lookbackMarketFeatures = LookbackData(tsParams.getLookbackSize(), list(chain(columns, compulsoryColumns, ['prediction'])))

    def getTsParams(self):
        return self.tsParams

    def getInstrument(self, instrumentId):
        if instrumentId not in self.__instrumentsDict:
            return None
        return self.__instrumentsDict[instrumentId]

    def getAllInstrumentsByInstrumentId(self):
        return self.__instrumentsDict

    def getLookbackMarketFeatures(self):
        return self.__lookbackMarketFeatures

    def getDataDf(self):
        return self.__lookbackMarketFeatures.getData()

    def createInstrumentFromUpdate(self, instrumentUpdate, tsParams):
        type = instrumentUpdate.getTypeOfInstrument()
        bookDataFeatures = instrumentUpdate.getBookDataFeatures()
        instrument = None
        if type == INSTRUMENT_TYPE_STOCK:
            stockInstrumentId = instrumentUpdate.getStockInstrumentId()
            instrument = StockInstrument(stockInstrumentId=stockInstrumentId,
                                         bookDataFeatures=bookDataFeatures,
                                         tsParams=tsParams)
        elif type == INSTRUMENT_TYPE_FUTURE:
            futureInstrumentId = instrumentUpdate.getFutureInstrumentId()
            expiryTime = instrumentUpdate.getExpiryTime()
            underlyingInstrumentId = instrumentUpdate.getUnderlyingInstrumentId()
            instrument = FutureInstrument(futureInstrumentId=futureInstrumentId,
                                          bookDataFeatures=bookDataFeatures,
                                          expiryTime=expiryTime,
                                          underlyingInstrumentId=underlyingInstrumentId,
                                          tsParams=tsParams)
        elif type == INSTRUMENT_TYPE_OPTION:
            optionInstrumentId = instrumentUpdate.getOptionInstrumentId()
            strikePrice = instrumentUpdate.getStrikePrice()
            optionType = instrumentUpdate.getOptionType()
            expiryTime = instrumentUpdate.getExpiryTime()
            underlyingInstrumentId = instrumentUpdate.getUnderlyingInstrumentId()
            instrument = OptionInstrument(optionInstrumentId=optionInstrumentId,
                                          bookDataFeatures=bookDataFeatures,
                                          strikePrice=strikePrice,
                                          optionType=optionType,
                                          expiryTime=expiryTime,
                                          underlyingInstrumentId=underlyingInstrumentId,
                                          tsParams=tsParams)
        return instrument

    def addInstrument(self, instrument):
        instrumentId = instrument.getInstrumentId()
        self.__instrumentsDict[instrumentId] = instrument

    def updateFeatures(self, timeOfUpdate):

        for instrumentId in self.__instrumentsDict:
            instrument = self.__instrumentsDict[instrumentId]
            instrument.updateFeatures(timeOfUpdate, self)

        currentMarketFeatures = {}
        self.__lookbackMarketFeatures.addData(timeOfUpdate, currentMarketFeatures)
        featureConfigs = self.tsParams.getMarketFeatureConfigs() + self.__compulsoryFeatureConfigs
        for featureConfig in featureConfigs:
            featureId = featureConfig.getFeatureId()
            featureKey = featureConfig.getFeatureKey()
            featureParams = featureConfig.getFeatureParams()
            featureCls = FeatureConfig.getClassForFeatureId(featureId)
            featureVal = featureCls.computeForMarket(featureParams=featureParams,
                                                     featureKey=featureKey,
                                                     currentMarketFeatures=currentMarketFeatures,
                                                     instrumentManager=self)
            currentMarketFeatures[featureKey] = featureVal
            self.__lookbackMarketFeatures.addFeatureVal(timeOfUpdate, featureKey, featureVal)
        currentMarketFeatures['prediction'] = self.tsParams.getPrediction(timeOfUpdate, currentMarketFeatures, self)
        self.__lookbackMarketFeatures.addFeatureVal(timeOfUpdate, 'prediction', currentMarketFeatures['prediction'])

        logInfo('Market Features: %s' % str(currentMarketFeatures))



from backtester.features.feature import Feature
from backtester.financial_fn import ma
import numpy as np


class DifferenceFeature(Feature):

    @classmethod
    def computeForLookbackData(cls, featureParams, featureKey, currentFeatures, lookbackDataDf):
        data = lookbackDataDf[featureParams['featureName']]
        if len(data.index) < featureParams['period'] :
            return 0
        return data[-1] - data[-featureParams['period']]



from backtester.instruments.instrument import Instrument
from backtester.instruments.future_instrument import FutureInstrument
from backtester.instruments.option_instrument import OptionInstrument
from backtester.instruments.stock_instrument import StockInstrument

__all__ = [
    'Instrument',
    'FutureInstrument',
    'OptionInstrument',
    'StockInstrument',
]



from backtester.executionSystem.simple_execution_system import SimpleExecutionSystem
from backtester.logger import *
import numpy as np


class SimpleExecutionSystemWithFairValue(SimpleExecutionSystem):
    def __init__(self, enter_threshold_deviation=0.07, exit_threshold_deviation=0.05, longLimit=10,
                 shortLimit=10, capitalUsageLimit=0, lotSize=1, limitType='L', price=''):
        super(SimpleExecutionSystemWithFairValue, self).__init__(enter_threshold=enter_threshold_deviation,
                                                                 exit_threshold=exit_threshold_deviation,
                                                                 longLimit=longLimit, shortLimit=shortLimit,
                                                                 capitalUsageLimit=capitalUsageLimit,
                                                                 lotSize=lotSize, limitType=limitType, price=price)

    def getBuySell(self, instrument, currentPredictions):
        instrumentId = instrument.getInstrumentId()
        try:
            currentPrice = instrument.getDataDf()[self.priceFeature].iloc[-1]
            fairValue = currentPredictions[instrumentId]
            currentDeviationFromPrediction = fairValue / currentPrice
            return -np.sign(currentDeviationFromPrediction)
        except KeyError:
            logError('You have specified FairValue Execution Type but Price Feature Key does not exist')

    def enterCondition(self, instrumentsManager, instrument, currentPredictions):
        instrumentId = instrument.getInstrumentId()
        try:
            currentPrice = instrument.getDataDf()[self.priceFeature].iloc[-1]
            fairValue = currentPredictions[instrumentId]
            currentDeviationFromPrediction = fairValue / currentPrice
            return np.abs(currentDeviationFromPrediction) > (self.enter_threshold)
        except KeyError:
            logError('You have specified FairValue Execution Type but Price Feature Key does not exist')

    def exitCondition(self, instrumentsManager, instrument, currentPredictions):
        instrumentId = instrument.getInstrumentId()
        try:
            currentPrice = instrument.getDataDf()[self.priceFeature].iloc[-1]
            fairValue = currentPredictions[instrumentId]
            currentDeviationFromPrediction = fairValue / currentPrice
            return np.abs(currentDeviationFromPrediction) < (self.exit_threshold)
        except KeyError:
            logError('You have specified FairValue Execution Type but Price Feature Key does not exist')

    def hackCondition(self, instrumentsManager):
        return False

    # def getExecutions(self, time, instrumentsManager, capital):
    # 	# TODO:
    # 	marketFeaturesDf = instrumentsManager.getDataDf()
    # 	currentMarketFeatures = marketFeaturesDf.iloc[-1]
    # 	currentPredictions = marketFeaturesDf['prediction'].iloc[-1]
    # 	logInfo(str(currentPredictions))
    # 	currentDeviationFromPredictions = {}
    # 	currentProbabilityPredictions = {}
    # 	for instrumentId in currentPredictions.keys():
    # 		instrument = instrumentsManager.getInstrument(instrumentId)
    # 		if instrument is None:
    # 			continue
    # 		try:
    # 			currentPrice = instrument.getDataDf()[self.priceFeature].iloc[-1]
    # 		except KeyError:
    # 			logError('You have specified FairValue Execution Type but Price Feature Key does not exist')

    # 		currentDeviationFromPrediction = currentPredictions[instrumentId]/currentPrice
    # 		currentProbabilityPredictions[instrumentId] = currentDeviationFromPrediction/2

    # 	executions = []
    # 	executions += self.exitPosition(instrumentsManager, currentProbabilityPredictions)
    # 	executions += self.enterPosition(instrumentsManager, currentProbabilityPredictions, capital)
    # 	return executions



from backtester.instruments.instrument import Instrument
from backtester.constants import *


class FutureInstrument(Instrument):

    def __init__(self, futureInstrumentId, bookDataFeatures, expiryTime, underlyingInstrumentId, tsParams):
        super(FutureInstrument, self).__init__(futureInstrumentId, bookDataFeatures, tsParams)
        self.__futureInstrumentId = futureInstrumentId
        self.__expiryTime = expiryTime
        self.__underlyingInstrumentId = underlyingInstrumentId

    def getInstrumentType(self):
        return INSTRUMENT_TYPE_FUTURE
    '''
    Returns datettime object for expiry of option
    '''

    def getExpiryTime(self):
        return self.__expiryTime

    def getUnderlyingInstrumentId(self):
        return self.__underlyingInstrumentId



import pandas as pd
import numpy as np
from backtester.plotter import generateGraph
from os import listdir
from os.path import isfile, join, basename
from backtester.metrics.metrics import Metrics
from backtester.logger import *

'''
Usage(to test in console):
>> pip install plotly
>> python
>> from plotter import plot
>> plot(r'./runLogs/runLog_20170613_201741', []) # plots every file in directory
>> plot(r'./runLogs/runLog_20170613_201741', ['ADBE_features.csv']) # ignores the files listed here
>> plot(r'./runLogs/runLog_20170613_201741/ADBE_features.csv', []) # plots only ADBE_features
Replace the appropriate timeStamps

TODO: 1) Support excluding columns for each files.
      2) Cleanup the html files generated from plotting/dont regenerate and recycle ones present.
      3) Provide a selector GUI to chose the files.
'''


def processResult(dir, marketFeatures, benchmark, stats, metricString, startingCapital, excludeFiles, shouldPlot):
    resultDict = {}
    if marketFeatures is not None and isfile(marketFeatures):
        df, benchmark_pnl = getDataReady(
            dir, marketFeatures, benchmark, startingCapital, True)
        if shouldPlot:
            logInfo('Generating %s' % marketFeatures)
            generateGraph(df, marketFeatures, metricString, benchmark_pnl)
        resultDict['metrics_values'] = stats.keys()
        resultDict['metrics'] = stats.values()
        resultDict['dates'] = df.index.values
        resultDict['pnl'] = df['pnl'].values
    else:
        resultDict['instrument_names'] = []
        resultDict['instrument_stats'] = []
        for fileName in listdir(dir):
            path = dir + '/' + fileName
            if (not isfile(path)) or (path in excludeFiles) or (fileName in excludeFiles):
                logInfo('excluding ', fileName)
                continue
            logInfo('Generating %s' % fileName)
            df, benchmark_pnl = getDataReady(
                dir, path, benchmark, startingCapital, False)
            if shouldPlot:
                logInfo('Generating %s' % fileName)
                generateGraph(df, path, fileName + ' ' + metricString, benchmark_pnl)
            resultDict['instrument_names'] += [fileName.split('_')[0]]
            resultDict['instrument_stats'] += [{'total_pnl': stats['Total Pnl(%)'], 'score': stats['Score']}]
    return resultDict

def getDataReady(dir, features, benchmark, startingCapital, market=True):
    df = pd.read_csv(features, engine='python',
                     index_col='time', parse_dates=True)
    if market:
        df['Returns(%)'] = 100 * (df['pnl'] / startingCapital)
        # metrics = Metrics(marketFeaturesDf=df)
        # metrics.calculateMarketMetrics(benchmark, price, startingCapital, dir)
        # # TODO create benchamrks later
        benchmark_pnl = None
    else:
        # metrics = Metrics(marketFeaturesDf=df)
        # metrics.calculateMetrics(price, startingCapital)
        benchmark_pnl = None
    # logInfo(stats, True)
    return df, benchmark_pnl



from backtester.features.feature import Feature
import math


class ExpMovingAverageFeature(Feature):

    @classmethod
    def computeForLookbackData(cls, featureParams, featureKey, currentFeatures, lookbackDataDf):
        data = lookbackDataDf[featureKey]
        if len(data.index) > 1:
            prev_ema = data[-2]
        else:
            prev_ema = currentFeatures[featureParams['featureName']]
        halflife = featureParams['period']
        alpha = 1 - math.exp(math.log(0.5) / halflife)
        avg = currentFeatures[featureParams['featureName']] * alpha + prev_ema * (1 - alpha)
        return avg



class DataSource(object):
    # emits list of instrument updates at the same time.
    # The caller needs to ensure all these updates are happening at the same time
    # emits [t1, [i1, i2, i3]], where i1, i2, i3 are updates happening at time t1
    def emitInstrumentUpdates(self):
        raise NotImplementedError



from backtester.instrumentUpdates.instrument_update import InstrumentUpdate
from backtester.constants import *


class OptionInstrumentUpdate(InstrumentUpdate):
    '''
    optionInstrumentId: symbol for option id
    timeOfUpdate: datetime object for time of update
    expiryTime: datettime object for expiry of option
    bookData: dictionary. keys which we support right now:
             bidPrice
             bidVolume
             askPrice
             askVolume
             volume
             openInterest
    '''
    def __init__(self, optionInstrumentId, tradeSymbol, timeOfUpdate, bookData, strikePrice, optionType, expiryTime, underlyingInstrumentId):
        super(OptionInstrumentUpdate, self).__init__(optionInstrumentId, tradeSymbol, timeOfUpdate, bookData)
        self.__optionInstrumentId = optionInstrumentId
        self.__expiryTime = expiryTime
        self.__strikePrice = strikePrice
        self.__optionType = optionType
        self.__underlyingInstrumentId = underlyingInstrumentId

    def getOptionInstrumentId():
        return self.__optionInstrumentId

    def getTypeOfInstrument(self):
        return INSTRUMENT_TYPE_OPTION

    def getExpiryTime(self):
        return self.__expiryTime

    def getStrikePrice(self):
        return self.__strikePrice

    def getOptionType(self):
        return self.__optionType

    def getUnderlyingInstrumentId(self):
        return self.__underlyingInstrumentId



from datetime import datetime, timedelta
import calendar
from backtester.instrumentUpdates import *
from backtester.constants import *
from backtester.dataSource.data_source import DataSource
import os.path
from backtester.dataSource.data_source_utils import groupAndSortByTimeUpdates

TYPE_LINE_UNDEFINED = 0
TYPE_LINE_BOOK_DATA = 1
TYPE_LINE_GREEK = 2  # not used anymore
TYPE_LINE_BOOK_OPTION = 3


def checkDate(lineItem):
    try:
        datetime.strptime(lineItem, '%Y/%m/%d')
        return True
    except ValueError:
        return False


def checkTimestamp(lineItem):
    return True


# Returns the type of lineItems
def validateLineItem(lineItems):
    if len(lineItems) < 4:
        return TYPE_LINE_UNDEFINED
    if checkDate(lineItems[0]) and checkTimestamp(lineItems[1]) and lineItems[2] == "Book":
        return TYPE_LINE_BOOK_DATA
    if checkDate(lineItems[0]) and checkTimestamp(lineItems[1]) and lineItems[2] == "Greek:":
        return TYPE_LINE_GREEK
    if len(lineItems) == 7 and lineItems[3] == '|':
        return TYPE_LINE_BOOK_OPTION
    return TYPE_LINE_UNDEFINED


def parseBookDataOptionLine(lineItems):
    if (len(lineItems) < 7):
        return None
    bidVol = float(lineItems[1])
    bidPrice = float(lineItems[2])
    askPrice = float(lineItems[4])
    askVol = float(lineItems[5])
    return {'bidVolume': bidVol,
            'bidPrice': bidPrice,
            'askPrice': askPrice,
            'askVolume': askVol}


def get_exp_date(trade_date):
    date = max(week[-4] for week in calendar.monthcalendar(trade_date.year, trade_date.month))
    if date >= trade_date.day:
        exp_date = datetime(year=trade_date.year, month=trade_date.month, day=date)
    else:
        if trade_date.month != 12:
            date = max(week[-4] for week in calendar.monthcalendar(trade_date.year, 1 + trade_date.month))
            exp_date = datetime(year=trade_date.year, month=1 + trade_date.month, day=date)
        else:
            date = max(week[-4] for week in calendar.monthcalendar(1 + trade_date.year, 1))
            exp_date = datetime(year=1 + trade_date.year, month=1, day=date)
    holiday_dates = ['20160706', '20160815', '20160905', '20160913', '20161011', '20161012', '20161031',
                     '20161114', '20170126', '20170224', '20170313', '20170404', '20170414', '20170501', '20170626']
    if datetime.strftime(exp_date, '%Y%m%d') in holiday_dates:
        exp_date = exp_date + timedelta(days=-1)
    return exp_date.replace(hour=15, minute=30)


class InstrumentsFromFile():
    def __init__(self, fileName, instrumentId, expiryTime):
        self.fileName = fileName
        self.instrumentId = instrumentId
        self.expiryTime = expiryTime
        self.currentInstrumentSymbol = None
        self.currentTimeOfUpdate = None
        self.currentBookData = None

    def processLine(self, line):
        lineItems = line.split()
        lineItemType = validateLineItem(lineItems)
        if (lineItemType == TYPE_LINE_BOOK_DATA):
            inst = None
            if self.currentInstrumentSymbol is not None:
                inst = FutureInstrumentUpdate(futureInstrumentId=self.instrumentId,
                                              tradeSymbol=self.currentInstrumentSymbol,
                                              timeOfUpdate=self.currentTimeOfUpdate,
                                              bookData=self.currentBookData,
                                              expiryTime=self.expiryTime,
                                              underlyingInstrumentId='NA')
            self.currentTimeOfUpdate = datetime.strptime(lineItems[0] + ' ' + lineItems[1], "%Y/%m/%d %H:%M:%S:%f")
            self.currentInstrumentSymbol = lineItems[4]
            self.currentBookData = None
            return inst
        elif(lineItemType == TYPE_LINE_BOOK_OPTION):
            parsedOption = parseBookDataOptionLine(lineItems)
            if self.currentBookData is None:
                self.currentBookData = parsedOption

    def processLinesIntoInstruments(self):
        with open(self.fileName, "r") as ins:
            instruments = []
            for line in ins:
                inst = self.processLine(line)
                if inst is not None:
                    instruments.append(inst)
            return instruments


class AuquanDataSource(DataSource):
    def __init__(self, folderName, instrumentIdsByType, startDateStr, endDateStr):
        self.startDate = datetime.strptime(startDateStr, "%Y/%m/%d")
        self.endDate = datetime.strptime(endDateStr, "%Y/%m/%d")
        self.folderName = folderName
        self.instrumentIdsByType = instrumentIdsByType
        self.currentDate = self.startDate

    def getFileName(self, instrumentType, instrumentId, date):
        dateStr = date.strftime("%Y%m%d")
        return '%s/%s/%s/%s_%s.txt' % (self.folderName, instrumentType, instrumentId, instrumentId, dateStr)

    def emitInstrumentUpdates(self):
        while (self.currentDate <= self.endDate):
            allInstrumentUpdates = []
            for instrumentType in self.instrumentIdsByType:
                instrumentIds = self.instrumentIdsByType[instrumentType]
                for instrumentId in instrumentIds:
                    fileName = self.getFileName(instrumentType, instrumentId, self.currentDate)
                    if not os.path.isfile(fileName):
                        continue
                    expiryTime = get_exp_date(self.currentDate)
                    fileHandler = InstrumentsFromFile(fileName=fileName, instrumentId=instrumentId, expiryTime=expiryTime)
                    instrumentUpdates = fileHandler.processLinesIntoInstruments()
                    allInstrumentUpdates = allInstrumentUpdates + instrumentUpdates
            groupedInstrumentUpdates = groupAndSortByTimeUpdates(allInstrumentUpdates)
            for timeOfUpdate, instrumentUpdates in groupedInstrumentUpdates:
                yield([timeOfUpdate, instrumentUpdates])
            self.currentDate = self.currentDate + timedelta(days=1)


if __name__ == "__main__":
    a = datetime.strptime('2017/06/30', "%Y/%m/%d")
    print(get_exp_date(a))



from backtester.logger import *


class PlacedOrder():
    def __init__(self, instrumentId, changeInPosition, timeOfExecution):
        self.__instrumentId = instrumentId
        self.__changeInPosition = changeInPosition
        self.__tradePrice = None
        self.__timeOfExecution = timeOfExecution

    def getInstrumentId(self):
        return self.__instrumentId

    def getChangeInPosition(self):
        return self.__changeInPosition

    def getTradePrice(self):
        if self.__tradePrice is None:
            logError('tradePrice is not set yet')
            return 0
        return self.__tradePrice

    def setTradePrice(self, tradePrice):
        self.__tradePrice = tradePrice

    def getTimeOfExecution(self):
        return self.__timeOfExecution


# TODO: For live orders, we need a way to track what orders we have placed but havent gotten
# a confirmation for. Also, need something for failure of orders.
class BaseOrderPlacer():

    '''
    instrumentExecutions: Array of InstrumentExecution
    '''
    def placeOrders(self, time, instrumentExecutions, instrumentsManager):
        raise NotImplementedError

    def emitPlacedOrders(self):
        raise NotImplementedError



from backtester.features.feature import Feature


class CapitalFeature(Feature):

    '''
    Computing for Instrument. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForInstrument(cls, featureParams, featureKey, currentFeatures, instrument, instrumentManager):

        currentPosition = instrument.getCurrentPosition()
        if len(instrument.getDataDf()['position']) <= 1:
            previousPosition = 0
        else:
            previousPosition = instrument.getDataDf()['position'][-2]
        currentPrice = currentFeatures[featureParams['price']]
        changeInCapital = (currentPosition - previousPosition) * currentPrice + currentFeatures[featureParams['fees']]
        return changeInCapital

    '''
    Computing for Market. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForMarket(cls, featureParams, featureKey, currentMarketFeatures, instrumentManager):
        changeInCapital = 0
        capitalDict = instrumentManager.getDataDf()[featureKey]
        if len(capitalDict) <= 1:
            return featureParams['initial_capital']
        capital = capitalDict.values[-2]
        allInstruments = instrumentManager.getAllInstrumentsByInstrumentId()
        for instrumentId in allInstruments:
            instrument = allInstruments[instrumentId]
            changeInCapital -= instrument.getDataDf()[featureKey][-1]
        return capital + changeInCapital



from backtester.features.feature import Feature
from backtester.financial_fn import ma
import numpy as np
import pandas as pd


class CrossSectionMomentumFeature(Feature):

    '''
    Computing for Instrument. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForInstrument(cls, featureParams, featureKey, currentFeatures, instrument, instrumentManager):      
        raise NotImplementedError
        return None

    '''
    Computing for Market. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForMarket(cls, featureParams, featureKey, currentMarketFeatures, instrumentManager):
        data = {}
        if 'instrumentIds' in featureParams:
            allInstruments = featureParams['instrumentIds']
        else:
            allInstruments = instrumentManager.getAllInstrumentsByInstrumentId()
        for instrumentId in allInstruments:
            instrument = allInstruments[instrumentId]
            data[instrumentId] = instrument.getDataDf()[featureParams['featureName']]
        df = pd.DataFrame(data)
        R = (df / df.shift(featureParams['period']))
        ranks = (R.T - R.T.mean()).T.mean()
        return ranks




from backtester.features.feature import Feature
import numpy as np
class ScoreRankFeature(Feature):

    @classmethod
    def computeForInstrument(cls, featureParams, featureKey, currentFeatures, instrument, instrumentManager):      
        raise NotImplementedError
        return None

    '''
    Computing for Market. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForMarket(cls, featureParams, featureKey, currentMarketFeatures, instrumentManager):
        score = 0
        scoreDict = instrumentManager.getDataDf()[featureKey]
        predictionKey = 'prediction'
        priceKey = 'close'
        if 'predictionKey' in featureParams:
            predictionKey = featureParams['predictionKey']
        if 'price' in featureParams:
           priceKey = featureParams['price']
        if len(scoreDict) < 1:
            return 0
        cumulativeScore = scoreDict.values[-1]
        allInstruments = instrumentManager.getAllInstrumentsByInstrumentId()
        returns = pd.Series(0, index = allInstruments.keys())
        for instrumentId in allInstruments:
            instrument = allInstruments[instrumentId]
            lookbackDataDf = instrument.getDataDf()
            returns[instrumentId] = lookbackDataDf[priceKey].iloc[-1]/lookbackDataDf[priceKey].iloc[-2]

        returns.dropna(inplace=True)
        rank = returns.rank(ascending=False)
        p = instrumentManager.getDataDf()[prediction].iloc[-1]
        score = ((rank - p ) * (rank - rank.mean())).abs().sum() 
        cumulativeScore -= score/len(allInstruments)
        return score




from backtester.features.feature import Feature


class ProfitLossRatioFeature(Feature):

    @classmethod
    def computeForLookbackData(cls, featureParams, featureKey, currentFeatures, lookbackDataDf):

        pnlKey = 'pnl'
        if 'pnlKey' in featureParams:
            pnlKey = featureParams['pnlKey']
        if len(lookbackDataDf) <= 1:
            prevData = {'totalLoss': 0, 'totalProfit': 0,
                        'countLoss': 0, 'countProfit': 0}
            prevPnl = 0
        else:
            prevData = lookbackDataDf[featureKey].iloc[-2]
            prevPnl = lookbackDataDf[pnlKey].iloc[-2]

        totalLoss = prevData['totalLoss']
        totalProfit = prevData['totalProfit']
        countLoss = prevData['countLoss']
        countProfit = prevData['countProfit']

        pnl = lookbackDataDf[pnlKey].iloc[-1] - prevPnl

        if (pnl > 0):
            totalProfit += pnl
            countProfit += 1
        elif (pnl < 0):
            totalLoss += -pnl
            countLoss += 1

        return {'totalLoss': totalLoss, 'totalProfit': totalProfit,
                'countLoss': countLoss, 'countProfit': countProfit}



from backtester.features.feature import Feature


class RatioMarketFeature(Feature):

    '''
    Computing for Instrument. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForInstrument(cls, featureParams, featureKey, currentFeatures, instrument, instrumentManager):
        feature1 = featureParams['featureName1']
        feature2 = featureParams['featureName2']
        if (instrument):
            return 0
        lookbackData = instrument.getDataDf()
        if len(lookbackData) < 1:
            return 1
        feature1 = lookbackData[featureParams['featureName1']].iloc[-1]
        feature2 = lookbackData[featureParams['featureName2']].iloc[-1]
        if feature2 == 0:
            return 0
        return feature2 / float(feature1)

    '''
    Computing for Market. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForMarket(cls, featureParams, featureKey, currentMarketFeatures, instrumentManager):
        feature = featureParams['featureName']
        instrumentId1 = featureParams['instrumentId1']
        instrument1 = instrumentManager.getInstrument(instrumentId1)
        instrumentId2 = featureParams['instrumentId2']
        instrument2 = instrumentManager.getInstrument(instrumentId2)
        if (instrument1 is None) or (instrument2 is None):
            return 0
        else:
            instrument1Price = instrument1.getDataDf()[feature].iloc[-1]
            instrument2Price = instrument2.getDataDf()[feature].iloc[-1]
            if instrument2Price == 0:
                return 0
            return instrument1Price / float(instrument2Price)



from backtester.instrumentUpdates.instrument_update import InstrumentUpdate
from backtester.instrumentUpdates.future_instrument_update import FutureInstrumentUpdate
from backtester.instrumentUpdates.option_instrument_update import OptionInstrumentUpdate
from backtester.instrumentUpdates.stock_instrument_update import StockInstrumentUpdate

__all__ = [
    'InstrumentUpdate',
    'FutureInstrumentUpdate',
    'OptionInstrumentUpdate',
    'StockInstrumentUpdate',
]



from backtester.features.feature import Feature
from backtester.logger import *
from backtester.features.argmax_feature import ArgMaxFeature
from backtester.features.argmin_feature import ArgMinFeature
from backtester.features.bollinger_bands_feature import BollingerBandsFeature
from backtester.features.capital_feature import CapitalFeature
from backtester.features.count_feature import CountFeature
from backtester.features.crossInstrument_correlation_feature import MovingInstrumentCorrelationFeature
from backtester.features.crossSectionMomentum_feature import CrossSectionMomentumFeature
from backtester.features.delay_feature import DelayFeature
from backtester.features.difference_feature import DifferenceFeature
from backtester.features.direction_feature import DirectionFeature
from backtester.features.exp_moving_average_feature import ExpMovingAverageFeature
from backtester.features.fees_feature import FeesFeature
from backtester.features.macd_feature import MACDFeature
from backtester.features.ma_ribbon_distance_feature import MARibbonDistanceFeature
from backtester.features.momentum_feature import MomentumFeature
from backtester.features.moving_average_feature import MovingAverageFeature
from backtester.features.moving_correlation import MovingCorrelationFeature
from backtester.features.moving_max import MovingMaximumFeature
from backtester.features.moving_min import MovingMinimumFeature
from backtester.features.moving_sdev_feature import MovingSDevFeature
from backtester.features.moving_sum import MovingSumFeature
from backtester.features.portfolio_value_feature import PortfolioValueFeature
from backtester.features.position_instrument_feature import PositionInstrumentFeature
from backtester.features.profitloss_feature import ProfitLossFeature
from backtester.features.rank_feature import RankFeature
from backtester.features.ratio_feature import RatioMarketFeature
from backtester.features.rsi_feature import RSIFeature
from backtester.features.scale_feature import ScaleFeature
from backtester.features.score_fairvalue_feature import ScoreFairValueFeature
from backtester.features.score_logloss_feature import ScoreLogLossFeature
from backtester.features.vwap_price_feature import VwapPriceInstrumentFeature
from backtester.features.score_problem1 import ProblemOneScore
from backtester.features.variance_feature import VarianceFeature
from backtester.features.maxcapitalusage_feature import MaxCapitalUsageFeature
from backtester.features.maxDrawdown_feature import MaxDrawdownFeature
from backtester.features.profitlossratio_feature import ProfitLossRatioFeature


featureIdToFeatureCls = {'argmax': ArgMaxFeature,
                         'argmin': ArgMinFeature,
                         'bollinger_bands': BollingerBandsFeature,
                         'capital': CapitalFeature,
                         'count': CountFeature,
                         'cross_sectional_momentum': CrossSectionMomentumFeature,
                         'cross_instrument_correlation': MovingInstrumentCorrelationFeature,
                         'delay': DelayFeature,
                         'difference': DifferenceFeature,
                         'direction': DirectionFeature,
                         'exponential_moving_average': ExpMovingAverageFeature,
                         'fees': FeesFeature,
                         'macd': MACDFeature,
                         'ma_ribbon_distance': MARibbonDistanceFeature,
                         'momentum': MomentumFeature,
                         'moving_average': MovingAverageFeature,
                         'moving_correlation': MovingCorrelationFeature,
                         'moving_max': MovingMaximumFeature,
                         'moving_min': MovingMinimumFeature,
                         'moving_sdev': MovingSDevFeature,
                         'moving_sum': MovingSumFeature,
                         'portfolio_value': PortfolioValueFeature,
                         'position': PositionInstrumentFeature,
                         'pnl': ProfitLossFeature,
                         'rank': RankFeature,
                         'ratio': RatioMarketFeature,
                         'rsi': RSIFeature,
                         'scale': ScaleFeature,
                         'score_fv': ScoreFairValueFeature,
                         'score_ll': ScoreLogLossFeature,
                         'vwap': VwapPriceInstrumentFeature,
                         'prob1_score': ProblemOneScore,
                         'pl_ratio': ProfitLossRatioFeature,
                         'maxDrawdown': MaxDrawdownFeature,
                         'variance': VarianceFeature,
                         'maxCapitalUsage': MaxCapitalUsageFeature
                         }


class FeatureConfig:

    customFeatureIdToFeatureCls = {}

    def __init__(self, configDict):
        if 'featureId' not in configDict:
            logError('featureId missing in market feature config dictionary %s', configDict['featureKey'])
        self.__featureIdentifier = configDict['featureId']

        if 'featureKey' in configDict:
            self.__featureKey = configDict['featureKey']
        else:
            self.__featureKey = self.__featureIdentifier

        if 'params' in configDict:
            self.__featureParams = configDict['params']
        else:
            self.__featureParams = {}

    def getFeatureKey(self):
        return self.__featureKey

    def getFeatureId(self):
        return self.__featureIdentifier

    def getFeatureParams(self):
        return self.__featureParams

    @classmethod
    def setupCustomFeatures(cls, customFeatureIdToFeatureCls):
        FeatureConfig.customFeatureIdToFeatureCls = customFeatureIdToFeatureCls

    @classmethod
    def getClassForFeatureId(cls, featureId):
        if featureId in FeatureConfig.customFeatureIdToFeatureCls:
            return FeatureConfig.customFeatureIdToFeatureCls[featureId]
        if featureId in featureIdToFeatureCls:
            return featureIdToFeatureCls[featureId]
        logError('%s not a valid feature Id. Use a predefined one or provide a custom implementation' % featureId)
        return Feature



import time
from backtester.logger import *
from backtester.instruments_manager import InstrumentManager
from datetime import datetime
from backtester.state_writer import StateWriter
from backtester.process_result import processResult
from backtester.metrics.metrics import Metrics


class TradingSystem:
    '''
    tsParams: Instance of TradingSystemParameters
    '''

    def __init__(self, tsParams):
        self.tsParams = tsParams
        self.portfolioValue = 0
        self.capital = 0
        self.instrumentManager = InstrumentManager(self.tsParams)
        self.startDate = None
        self.featuresUpdateTime = None
        self.totalTimeUpdating = 0  # for tracking perf
        self.totalUpdates = 0
        self.dataParser = None
        self.executionSystem = None
        self.orderPlacer = None
        self.stateWriter = StateWriter('runLogs', datetime.strftime(datetime.now(), '%Y%m%d_%H%M%S'))

    def processInstrumentUpdates(self, timeOfUpdate, instrumentUpdates, onlyAnalyze=False):
        # Process instrument updates first
        for instrumentUpdate in instrumentUpdates:
            instrumentIdToUpdate = instrumentUpdate.getInstrumentId()
            instrumentToUpdate = self.instrumentManager.getInstrument(instrumentIdToUpdate)
            # if not present try to create an instrument from this update first.
            if instrumentToUpdate is None:
                instrumentToUpdate = self.instrumentManager.createInstrumentFromUpdate(instrumentUpdate, self.tsParams)
                if instrumentToUpdate is None:
                    return
                self.instrumentManager.addInstrument(instrumentToUpdate)
            instrumentToUpdate.update(instrumentUpdate)
        # update positions of placed orders
        for placedOrder in self.orderPlacer.emitPlacedOrders(timeOfUpdate, self.instrumentManager):
            self.processPlacedOrder(placedOrder)
        # Then we try to calculate features.
        self.tryUpdateFeaturesAndExecute(timeOfUpdate, onlyAnalyze)

    def processPlacedOrder(self, placedOrder):
        instrumentId = placedOrder.getInstrumentId()
        changeInPosition = placedOrder.getChangeInPosition()
        placedInstrument = self.instrumentManager.getInstrument(instrumentId)
        tradePrice = placedOrder.getTradePrice()
        placedInstrument.updatePositionAtPrice(changeInPosition, tradePrice)

    def tryUpdateFeaturesAndExecute(self, timeOfUpdate, onlyAnalyze=False):
        # TODO: Fix this to run independently of time of update of instrument and at regular frequency updates
        shouldUpdateFeatures = False
        if self.featuresUpdateTime is None:
            shouldUpdateFeatures = True
        elif timeOfUpdate >= (self.featuresUpdateTime + self.tsParams.getFrequencyOfFeatureUpdates()):
            shouldUpdateFeatures = True
        if shouldUpdateFeatures:
            self.featuresUpdateTime = timeOfUpdate
            self.updateFeatures(timeOfUpdate)
            if not onlyAnalyze:
                instrumentsToExecute = self.getInstrumentsToExecute(timeOfUpdate)
                self.orderPlacer.placeOrders(timeOfUpdate, instrumentsToExecute, self.instrumentManager)
                self.portfolioValue = self.instrumentManager.getDataDf()['portfolio_value'][-1]  # TODO: find a better way to get this value
                self.capital = self.instrumentManager.getDataDf()['capital'][-1]  # TODO: find a better way to get this value

            self.saveCurrentState()

    def updateFeatures(self, timeOfUpdate):
        # tracking perf
        start = time.time()
        # if self.totalUpdates > 0:
        # Dont call for the first time
        self.instrumentManager.updateFeatures(timeOfUpdate)
        end = time.time()
        diffms = (end - start) * 1000
        self.totalTimeUpdating = self.totalTimeUpdating + diffms
        self.totalUpdates = self.totalUpdates + 1
        logInfo('Update: %d, Time: %.2f, Average: %.2f' % (self.totalUpdates, diffms, self.totalTimeUpdating / self.totalUpdates))

    def getInstrumentsToExecute(self, time):
        return self.executionSystem.getExecutions(time, self.instrumentManager, self.capital)

    def saveCurrentState(self):
        self.stateWriter.writeCurrentState(self.instrumentManager)

    def closePositions(self, timeOfUpdate):
        instrumentsToExecute = self.executionSystem.exitPosition(timeOfUpdate, self.instrumentManager, [], True)
        self.orderPlacer.placeOrders(timeOfUpdate, instrumentsToExecute, self.instrumentManager)
        for placedOrder in self.orderPlacer.emitPlacedOrders(timeOfUpdate, self.instrumentManager):
            self.processPlacedOrder(placedOrder)
        self.updateFeatures(timeOfUpdate)
        self.saveCurrentState()

    def getFinalMetrics(self, dateBounds, shouldPlotFeatures=True):
        allInstruments = self.instrumentManager.getAllInstrumentsByInstrumentId()
        resultDict = {}
        for instrumentId in allInstruments:
            instrument = allInstruments[instrumentId]
            lookbackData = instrument.getDataDf()
            metrics = Metrics(marketFeaturesDf=lookbackData)
            metrics.calculateMetrics(self.tsParams.getPriceFeatureKey(), self.tsParams.getStartingCapital())
            stats = metrics.getMetrics()
            metricString = metrics.getMetricsString()
            logInfo(metricString, True)
            resultDict.update(processResult(self.stateWriter.getFolderName(), None, None,
                 stats, metricString, self.tsParams.getStartingCapital(), [self.stateWriter.getMarketFeaturesFilename()], shouldPlotFeatures))
        metrics = Metrics(marketFeaturesDf=self.instrumentManager.getDataDf())
        metrics.calculateMarketMetrics(None, self.tsParams.getPriceFeatureKey(), self.tsParams.getStartingCapital(), dateBounds)
        stats = metrics.getMetrics()
        metricString = metrics.getMarketMetricsString()
        logInfo(metricString, True)
        # Hack to always plot market
        shouldPlotMarket = True
        resultDict.update(processResult(self.stateWriter.getFolderName(), self.stateWriter.getMarketFeaturesFilename(),
             self.tsParams.getBenchmark(), stats, metricString, self.tsParams.getStartingCapital(), [], shouldPlotMarket))
        return resultDict

    def initialize(self):
        self.dataParser = self.tsParams.getDataParser()
        self.executionSystem = self.tsParams.getExecutionSystem()
        self.orderPlacer = self.tsParams.getOrderPlacer()
        self.portfolioValue = self.tsParams.getStartingCapital()
        self.capital = self.tsParams.getStartingCapital()

    def startTrading(self, onlyAnalyze=False, shouldPlot=True):
        # TODO: Figure out a good way to handle order parsers with live data later on.
        self.initialize()
        groupedInstrumentUpdates = self.dataParser.emitInstrumentUpdates()

        for timeOfUpdate, instrumentUpdates in groupedInstrumentUpdates:
            # logInfo('TimeOfUpdate: %s TradeSymbol: %s' % (instrumentUpdate.getTimeOfUpdate(), instrumentUpdate.getTradeSymbol()))
            print(timeOfUpdate)
            if self.startDate is None:
                self.startDate = timeOfUpdate
            self.processInstrumentUpdates(timeOfUpdate, instrumentUpdates, onlyAnalyze)
            if not onlyAnalyze and self.portfolioValue < 0:
                logError('Trading will STOP - OUT OF MONEY!!!!')
                break

        if not onlyAnalyze:
            self.closePositions(timeOfUpdate)

        self.stateWriter.closeStateWriter()

        return self.getFinalMetrics([self.startDate, timeOfUpdate], shouldPlot)



def log(msg, verbose):
    if verbose:
        print(msg)


def logError(msg, verbose=True):
    log('Error: ' + msg, verbose)


def logInfo(msg, verbose=False):
    log('Info: ' + msg, verbose)


def logWarn(msg, verbose=True):
    log('Warn: ' + msg, verbose)



from backtester.features.feature import Feature

class CountFeature(Feature):

    @classmethod
    def computeForLookbackData(cls, featureParams, featureKey, currentFeatures, lookbackDataDf):
        data = lookbackDataDf[featureKey]
        if len(data) <= 1:
            return 0
        return data.iloc[-2]+1



from backtester.instruments.instrument import Instrument
from backtester.constants import *


class OptionInstrument(Instrument):

    def __init__(self, optionInstrumentId, bookDataFeatures, strikePrice, optionType, expiryTime, underlyingInstrumentId, tsParams):
        super(OptionInstrument, self).__init__(optionInstrumentId, bookDataFeatures, tsParams)
        self.__optionInstrumentId = optionInstrumentId
        self.__strikePrice = strikePrice
        self.__optionType = optionType
        self.__expiryTime = expiryTime
        self.__underlyingInstrumentId = underlyingInstrumentId

    def getInstrumentType(self):
        return INSTRUMENT_TYPE_OPTION

    def getOptionInstrumentId():
        return self.__optionInstrumentId

    '''
    Returns datettime object for expiry of option
    '''
    def getExpiryTime(self):
        return self.__expiryTime

    def getStrikePrice(self):
        return self.__strikePrice

    def getOptionType(self):
        return self.__optionType

    def getUnderlyingInstrumentId(self):
        return self.__underlyingInstrumentId



from backtester.trading_system_parameters import TradingSystemParameters
from datetime import timedelta
from backtester.dataSource.quant_quest_data_source import QuantQuestDataSource
from backtester.executionSystem.QQ_execution_system import QQExecutionSystem
from backtester.orderPlacer.backtesting_order_placer import BacktestingOrderPlacer
from backtester.constants import *


class FairValueTradingParams(TradingSystemParameters):

    def __init__(self, problem1Solver):
        self.__problem1Solver = problem1Solver
        super(FairValueTradingParams, self).__init__()

    def getStartingCapital(self):
        instrumentIds = self.__problem1Solver.getSymbolsToTrade()
        if instrumentIds and len(instrumentIds) > 0:
            return len(instrumentIds) * 10000
        return 1000000

    '''
    Returns an instance of class DataParser. Source of data for instruments
    '''

    def getDataParser(self):
        instrumentIds = self.__problem1Solver.getSymbolsToTrade()
        return QuantQuestDataSource(cachedFolderName='historicalData/',
                                    dataSetId=self.__problem1Solver.getTrainingDataSet(),
                                    instrumentIds=instrumentIds)

    '''
    Returns a timedetla object to indicate frequency of updates to features
    Any updates within this frequncy to instruments do not trigger feature updates.
    Consequently any trading decisions that need to take place happen with the same
    frequency
    '''

    def getFrequencyOfFeatureUpdates(self):
        return timedelta(0, 30)  # minutes, seconds

    def getBenchmark(self):
        return None

    '''
    This is a way to use any custom features you might have made.
    Returns a dictionary where
    key: featureId to access this feature (Make sure this doesnt conflict with any of the pre defined feature Ids)
    value: Your custom Class which computes this feature. The class should be an instance of Feature
    Eg. if your custom class is MyCustomFeature, and you want to access this via featureId='my_custom_feature',
    you will import that class, and return this function as {'my_custom_feature': MyCustomFeature}
    '''

    def getCustomFeatures(self):
        return self.__problem1Solver.getCustomFeatures()

    '''
    Returns a dictionary with:
    key: string representing instrument type. Right now INSTRUMENT_TYPE_OPTION, INSTRUMENT_TYPE_STOCK, INSTRUMENT_TYPE_FUTURE
    value: Array of instrument feature config dictionaries
        feature config Dictionary has the following keys:
        featureId: a string representing the type of feature you want to use
        featureKey: {optional} a string representing the key you will use to access the value of this feature.
                    If not present, will just use featureId
        params: {optional} A dictionary with which contains other optional params if needed by the feature
    Example:
    positionConfigDict = {'featureId': 'position'}
    vwapConfigDict = {'featureKey': 'price',
                          'featureId': 'vwap'}
    movingAvg_30Dict = {'featureKey': 'mv_avg_30',
                          'featureId': 'moving_average',
                          'params': {'days': 30}}
    movingAvg_90Dict = {'featureKey': 'mv_avg_90',
                          'featureId': 'moving_average',
                          'params': {'days': 90}}
    return {INSTRUMENT_TYPE_FUTURE: [positionConfigDict, vwapConfigDict],
            INSTRUMENT_TYPE_STOCK: [positionConfigDict, movingAvg_30Dict, movingAvg_90Dict]}

    For each future instrument, you will have features keyed by position and price.
    For each stock instrument, you will have features keyed by position, mv_avg_30, mv_avg_90
    '''

    def getInstrumentFeatureConfigDicts(self):
        stockFeatureConfigs = self.__problem1Solver.getFeatureConfigDicts()
        scoreDict = {'featureKey': 'score',
                           'featureId': 'prob1_score',
                           'params': {'predictionKey': 'prediction',
                                      'price': 'FairValue'}}
        sdevDictForExec = {'featureKey': 'sdev_5_for_exec',
                           'featureId': 'moving_sdev',
                           'params': {'period': 5,
                                      'featureName': 'basis'}}
        return {INSTRUMENT_TYPE_STOCK: stockFeatureConfigs + [sdevDictForExec, scoreDict]}

    '''
    Returns an array of market feature config dictionaries
        market feature config Dictionary has the following keys:
        featureId: a string representing the type of feature you want to use
        featureKey: a string representing the key you will use to access the value of this feature.this
        params: A dictionary with which contains other optional params if needed by the feature
    '''

    def getMarketFeatureConfigDicts(self):
        # ADD RELEVANT FEATURES HERE

        # customFeatureDict = {'featureKey': 'custom_mrkt_feature',
        #                      'featureId': 'my_custom_mrkt_feature',
        #                      'params': {'param1': 'value1'}}
        scoreDict = {'featureKey': 'score',
                     'featureId': 'prob1_score',
                     'params': {'price': 'FairValue',
                                'instrument_score_feature': 'score',
                                'benchmark_score_feature': 'benchmark_score'}}
        return [scoreDict]

    '''
    A function that returns your predicted value based on your heuristics.
    If you are just trading one asset like a stock, it could be the predicted value of the stock.
    If you are doing pair trading, the prediction could be the difference in the prices of the stocks.
    Arguments:
    time - When this prediction is being calculated
    currentMarketFeatures - Dictionary of market features which have been calculated at this update cycle.
    instrumentManager - Holder for all instruments and everything else if you need.
    '''

    def getPrediction(self, time, currentMarketFeatures, instrumentManager):
        instrumentIds = instrumentManager.getAllInstrumentsByInstrumentId()
        predictions = {}
        for ids in instrumentIds:
            instrument = instrumentManager.getInstrument(ids)
            predictions[ids] = self.__problem1Solver.getFairValue(time, instrument, instrumentManager)
        return predictions

    '''
    Returns the type of execution system we want to use. Its an implementation of the class ExecutionSystem
    It converts prediction to intended positions for different instruments.
    '''

    def getExecutionSystem(self):
        return QQExecutionSystem(basisEnter_threshold=0.5, basisExit_threshold=0.1,
                                 basisLongLimit=10000, basisShortLimit=10000,
                                 basisCapitalUsageLimit=0.05, basisLotSize=100,
                                 basisLimitType='L', basis_thresholdParam='sdev_5_for_exec',
                                 price=self.getPriceFeatureKey())

    '''
    Returns the type of order placer we want to use. its an implementation of the class OrderPlacer.
    It helps place an order, and also read confirmations of orders being placed.
    For Backtesting, you can just use the BacktestingOrderPlacer, which places the order which you want, and automatically confirms it too.
    '''

    def getOrderPlacer(self):
        return BacktestingOrderPlacer()

    '''
    Returns the amount of lookback data you want for your calculations. The historical market features and instrument features are only
    stored upto this amount.
    This number is the number of times we have updated our features.
    '''

    def getLookbackSize(self):
        return 90

    def getPriceFeatureKey(self):
        return 'basis'



from backtester.features.feature import Feature
import numpy as np


class FeesFeature(Feature):

    @classmethod
    def computeForLookbackData(cls, featureParams, featureKey, currentFeatures, lookbackDataDf):
        raise NotImplementedError
        return None

    @classmethod
    def computeForInstrument(cls, featureParams, featureKey, currentFeatures, instrument, instrumentManager):
        positionDict = instrument.getDataDf()['position']
        if len(positionDict) < 1:
            return 0
        currentPrice = currentFeatures[featureParams['price']]
        feesDict = featureParams['feesDict']
        currentPosition = instrument.getCurrentPosition()
        previousPosition = 0 if len(positionDict) < 2 else positionDict.iloc[-2]
        changeInPosition = currentPosition - previousPosition
        fees = np.abs(changeInPosition) * feesDict[np.sign(changeInPosition)]
        return fees

    @classmethod
    def computeForMarket(cls, featureParams, featureKey, currentMarketFeatures, instrumentManager):
        raise NotImplementedError
        return None



from backtester.features.feature import Feature
from backtester.financial_fn import ma
import numpy as np

# Average of feature name over some number of previous data points including current.
# number of data points specified by user

class MovingAverageFeature(Feature):

    @classmethod
    def computeForLookbackData(cls, featureParams, featureKey, currentFeatures, lookbackDataDf):
        data = lookbackDataDf[featureParams['featureName']]
        avg = data[-featureParams['period']:].mean()
        if len(data) < 1:
        	return 0
        return avg



from backtester.features.feature import Feature


class VwapPriceInstrumentFeature(Feature):

    '''
    Computing for Instrument. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForInstrument(cls, featureParams, featureKey, currentFeatures, instrument, instrumentManager):
        bookData = instrument.getCurrentBookData()
        totalVolume = (bookData['askVolume'] + bookData['bidVolume'])
        if totalVolume > 0:
            vwap = ((bookData['askPrice'] * bookData['askVolume']) + (bookData['bidPrice'] *
                                                                      bookData['bidVolume'])) / totalVolume
            return vwap
        else:
            return 0

    '''
    Computing for Market. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForMarket(cls, featureParams, featureKey, currentMarketFeatures, instrumentManager):
        raise NotImplementedError



from backtester.instrumentUpdates import *
from backtester.constants import *
from backtester.logger import *
from backtester.dataSource.data_source import DataSource
import os
import os.path
import requests
import re
from time import mktime as mktime
from itertools import groupby


def getCookieForYahoo(instrumentId):
    """Returns a tuple pair of cookie and crumb used in the request"""
    url = 'https://finance.yahoo.com/quote/%s/history' % (instrumentId)
    r = requests.get(url)
    txt = r.content
    cookie = r.cookies['B']
    pattern = re.compile('.*"CrumbStore":\{"crumb":"(?P<crumb>[^"]+)"\}')

    for line in txt.splitlines():
        m = pattern.match(line.decode("utf-8"))
        if m is not None:
            crumb = m.groupdict()['crumb']
            crumb = crumb.replace(u'\\u002F', '/')
    return cookie, crumb  # return a tuple of crumb and cookie


def downloadFileFromYahoo(startDate, endDate, instrumentId, fileName, event='history'):
    logInfo('Downloading %s' % fileName)
    cookie, crumb = getCookieForYahoo(instrumentId)
    start = int(mktime(startDate.timetuple()))
    end = int(mktime(endDate.timetuple()))
    url = 'https://query1.finance.yahoo.com/v7/finance/download/%s?period1=%s&period2=%s&interval=1d&events=%s&crumb=%s' % (instrumentId, start, end, event, crumb)
    data = requests.get(url, cookies={'B': cookie})
    with open(fileName, 'w') as f:
        f.write(data.content)


'''
Takes list of instruments.
Outputs them grouped by and sorted by time:
ie [[t1, [i1,i2,i3]],
    [t2, [i4]],
    [t3, [i5, i6]] ], where t1<t2<t3
'''
def groupAndSortByTimeUpdates(instrumentUpdates):
    instrumentUpdates.sort(key=lambda x: x.getTimeOfUpdate())
    groupedInstruments = []
    # groupby only works on already sorted elements, so we sorted first
    for timeOfUpdate, sameTimeInstruments in groupby(instrumentUpdates, lambda x: x.getTimeOfUpdate()):
        instruments = []
        for sameTimeInstrument in sameTimeInstruments:
            instruments.append(sameTimeInstrument)
        groupedInstruments.append([timeOfUpdate, instruments])
    return groupedInstruments



__all__ = ["accuracy_feature", "argmax_feature", "argmin_feature", "bollinger_bands_feature", "candle_features_1", "candle_features_2", "candle_features_3", "candle_features_4",
           "candle_features_5", "candle_features_6", "candle_features_7", "capital_feature", "count_feature", "crossInstrument_correlation_feature",
           "crossSectionMomentum_feature", "delay_feature", "difference_feature", "exp_moving_average_feature", "feature_config", "feature", "fees_feature",
           "ma_ribbon_distance_feature", "macd_feature", "maxcapitalusage_feature", "maxDrawdown_feature", "momentum_feature", "moving_average_feature", "moving_correlation",
           "moving_max", "moving_min", "moving_sdev_feature", "moving_sum", "portfolio_value_feature", "position_instrument_feature", "profitability_feature", "profitloss_feature",
           "profitlossratio_feature", "rank_feature", "ratio_feature", "rsi_feature", "scale_feature", "score_fairvalue_feature", "score_logloss_feature",
           "score_problem1", "score_rank_feature", "variance_feature", "vwap_price_feature"]



from backtester.features.feature import Feature
from backtester.financial_fn import ma
import numpy as np

# Correlation between two instruments over some number of data points specified by user.

class MovingCorrelationFeature(Feature):

    @classmethod
    def computeForInstrument(cls, featureParams, featureKey, currentFeatures, instrument, instrumentManager):
        lookbackInstrumentFeaturesDf = instrument.getDataDf()
        x = lookbackInstrumentFeaturesDf[featureParams['series1']]
        y = lookbackInstrumentFeaturesDf[featureParams['series2']]

        if (len(x) < 1) or (len(y) < 1):
        	return 0
        return round(x.rolling(featureParams['period']).corr(y)[-1], 3)

    @classmethod
    def computeForMarket(cls, featureParams, featureKey, currentMarketFeatures, instrumentManager):
        lookbackMarketFeaturesDf = instrumentManager.getDataDf()
        x = lookbackMarketFeaturesDf[featureParams['series1']]
        y = lookbackMarketFeaturesDf[featureParams['series2']]

        if (len(x) < 1) or (len(y) < 1):
        	return 0
        return round(x.rolling(featureParams['period']).corr(y)[-1], 3)



import pandas as pd
from collections import deque


class LookbackData:
    def __init__(self, size, columns):
        self.__size = size
        self.__columns = columns
        self.__storedData = deque([])
        self.__times = deque([])
        self.__data = pd.DataFrame(data=list(self.__storedData), columns=self.__columns, index=list(self.__times))
        # self.__data = pd.DataFrame(data=self.__storedData, columns=self.__columns, index=self.__times)
        # self.__storedData = pd.DataFrame(columns=columns)

    def addData(self, timeOfUpdate, data):
        self.__storedData.append(data)
        self.__times.append(timeOfUpdate)
        if len(self.__storedData) > self.__size:
            self.__storedData.popleft()
            self.__times.popleft()
        self.__data = pd.DataFrame(data=list(self.__storedData), columns=self.__columns, index=list(self.__times))
        # self.__storedData.loc[timeOfUpdate] = data

    '''
    returns a pandas dataframe index is time.
    '''
    def getData(self):
        return self.__data

    def getLastData(self):
        return self.__data.iloc[-1]

    def addFeatureVal(self, timeOfUpdate, featureKey, featureVal):
        # have to do this because in case featureVal is a dictionary and we have already set the first row
        # for this column to Nan, it raises an error if you try to update the cell which has value Nan
        # to dictionary.
        # only need to do it once for the first update
        if len(self.__data) == 1 and (isinstance(featureVal, dict)):
            self.__data[featureKey] = self.__data[featureKey].astype(object)
        self.__data.set_value(timeOfUpdate, featureKey, featureVal)



from backtester.features.feature import Feature
import numpy as np


class ProblemOneScore(Feature):

    @classmethod
    def computeForInstrument(cls, featureParams, featureKey, currentFeatures, instrument, instrumentManager):
        lookbackDataDf = instrument.getDataDf()
        predictionKey = 'prediction'
        price = 'close'
        countKey = 'count'
        if len(lookbackDataDf) < 1 or instrumentManager is None:
            return 0
        lookbackMarketDataDf = instrumentManager.getDataDf()
        if 'predictionKey' in featureParams:
            predictionKey = featureParams['predictionKey']
        if 'price' in featureParams:
            price = featureParams['price']
        if 'countKey' in featureParams:
            countKey = featureParams['countKey']
        if len(lookbackMarketDataDf) < 1:
            # first iteration
            return 0
        predictionDict = lookbackMarketDataDf[predictionKey].iloc[-1]
        # lookback market data is not updated yet since we are in instrument updates
        # therefore the count is previous count.
        prevCount = lookbackMarketDataDf[countKey].iloc[-1]
        if len(predictionDict) == 0:
            return 0
        prevData = lookbackDataDf[featureKey].iloc[-2]

        temp = (prevCount) * (prevData**2)

        sqError = (predictionDict[instrument.getInstrumentId()] - lookbackDataDf[price].iloc[-2])**2
        temp = (temp + sqError)
        # print(currentFeatures[price], lookbackDataDf[price].iloc[-1])
        return np.sqrt(float(temp) / float(prevCount + 1))

    '''
    Computing for Market. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForMarket(cls, featureParams, featureKey, currentMarketFeatures, instrumentManager):
        score = 0
        scoreDict = instrumentManager.getDataDf()[featureKey]
        scoreKey = 'score'
        normalizationKey = 'benchmark_score'
        if 'instrument_score_feature' in featureParams:
            scoreKey = featureParams['instrument_score_feature']
        if 'benchmark_score_feature' in featureParams:
            normalizationKey = featureParams['benchmark_score_feature']
        if len(scoreDict) <= 1:
            return 0
        cumulativeScore = scoreDict.values[-2]
        allInstruments = instrumentManager.getAllInstrumentsByInstrumentId()
        for instrumentId in allInstruments:
            instrument = allInstruments[instrumentId]
            lookbackDataDf = instrument.getDataDf()
            if lookbackDataDf[normalizationKey].iloc[-1] == 0:
                score += 1
            else:
                score += lookbackDataDf[scoreKey].iloc[-1] / (lookbackDataDf[normalizationKey].iloc[-1] / 1000)
        cumulativeScore += score / len(allInstruments)
        return score / float(len(allInstruments))



import numpy as np
import pandas as pd
from pandas.tseries.offsets import BDay
from functools import partial
from pandas.tseries.frequencies import to_offset


class Metrics():
    def __init__(self, marketFeaturesDf):
        self.__marketFeaturesDf = marketFeaturesDf
        self.__stats = {}

    def getMarketMetricsString(self):
        # TODO add the snippet back once benchmark is fixed.
        # + ' Benchmark: %0.2f%% ' % (100 * self.__stats['Base Return(%)']) \
        str = \
            ' Total Pnl: %0.2f%% ' % (100 * self.__stats['Total Pnl(%)']) \
            + ' Score: %0.2f ' % self.__stats['Score'] \
            + ' Max Drawdown: %0.2f%% ' % (100 * self.__stats['Max Drawdown(%)']) \
            + ' RoC: %0.2f%% ' % (100 * self.__stats['RoC(%)']) \
            + ' P/L Ratio: %0.2f ' % self.__stats['Profit/Loss Ratio'] \
            + ' Accuracy: %0.2f ' % self.__stats['Accuracy']
        if self.__stats['Trading Days'] > 252:
            str = str \
                + ' Ann. Return: %0.2f%% ' % (100 * self.__stats['Annual Return(%)']) \
                + ' Ann. Vol: %0.2f%% ' % (100 * self.__stats['Annual Vol(%)']) \
                + ' Sharpe Ratio: %0.2f ' % self.__stats['Sharpe Ratio'] \
                # + 'Log Loss         : %0.2f'%self.__stats['Log Loss']
        return str

    def getMetricsString(self):
        # Add back below once benchmark support
        # + ' Benchmark: %0.2f%% ' % (100 * self.__stats['Base Return(%)']) \
        return \
            ' Total Pnl: %0.2f%% ' % (100 * self.__stats['Total Pnl(%)']) \
            + ' Score: %0.2f ' % self.__stats['Score'] \
            + ' Profit/Loss Ratio: %0.2f ' % self.__stats['Profit/Loss Ratio'] \
            + ' Accuracy: %0.2f ' % self.__stats['Accuracy']
        # + 'Log Loss         : %0.2f'%self.__stats['Log Loss']

    def getMetrics(self):
        return self.__stats

    def getDf(self):
        return self.__marketFeaturesDf

    def round(self, t, freq):
        freq = to_offset(freq)
        return pd.Timestamp((t.value // freq.delta.value) * freq.delta.value)

    def resampleData(self, series, period):
        return series.groupby(partial(self.round, freq=period))
        # series.resample(period)

    def calculateMarketMetrics(self, baseSymbol, priceFeature, startingCapital, dateBounds):

        stats = {}
        df = self.__marketFeaturesDf

        # total_pnl = self.resampleData(
        #     self.__marketFeaturesDf['pnl'], '1D').last()  # TODO change 'pnl'
        # portfolioValue = self.resampleData(
        #     self.__marketFeaturesDf['portfolio_value'], '1D').last()  # TODO change portfolio_value
        # total_pnl.dropna(inplace=True)
        # portfolioValue.dropna(inplace=True)
        total_days = len(pd.date_range(dateBounds[0], dateBounds[1], freq=BDay()))
        total_return = df['pnl'].iloc[- 1] / float(startingCapital)

        benchmark = self.getBenchmarkData(None, priceFeature, '')
        stats['Trading Days'] = total_days
        stats['Total Pnl(%)'] = total_return
        stats['Annual Return(%)'] = self.annualized_return(
            total_return, total_days)
        if benchmark is not None:
            stats['Base Return(%)'] = self.annualized_return(
                benchmark['total_return'], total_days)
        stats['Annual Vol(%)'] = self.annual_vol(df['variance'].iloc[-1], startingCapital)
        # stats['Beta'] = self.beta(daily_return,benchmark['daily_returns'])
        stats['Sharpe Ratio'] = self.sharpe_ratio(stats['Annual Return(%)'], stats['Annual Vol(%)'])
        stats['RoC(%)'] = self.roc(df['pnl'].iloc[- 1], df['capitalUsage'].iloc[-1])
        stats['Score'] = df['score'].iloc[-1]
        stats['Max Drawdown(%)'] = self.max_drawdown(df['maxDrawdown'].iloc[-1], startingCapital)
        stats['Profit/Loss Ratio'] = self.profit_factor(df['pl_ratio'].iloc[-1])
        stats['Accuracy'] = self.accuracy(df['pl_ratio'].iloc[-1])
        # TODO change reference to score
        if 'score' in self.__marketFeaturesDf.columns:
            stats['Score'] = self.__marketFeaturesDf['score'].iloc[-1]
        # stats['Log Loss']=logLoss(daily_return)
        self.__stats = stats

    def calculateMetrics(self, priceFeature, startingCapital):

        stats = {}

        # total_pnl = self.resampleData(
        #     self.__marketFeaturesDf['pnl'], '1D').last()
        # price = self.resampleData(
        #     self.__marketFeaturesDf[priceFeature], '1D').last()
        # total_pnl.dropna(inplace=True)
        # price.dropna(inplace=True)
        df = self.__marketFeaturesDf
        total_days = len(pd.date_range(df.index[0], df.index[-1], freq=BDay()))
        total_return = df['pnl'].iloc[- 1] / float(startingCapital)
        base_return = df[priceFeature].iloc[total_days - 1] / df[priceFeature].iloc[0] - 1

        stats['Total Pnl(%)'] = total_return
        stats['Base Return(%)'] = self.annualized_return(
            base_return, total_days)
        stats['Score'] = df['score'].iloc[-1]
        stats['Profit/Loss Ratio'] = self.profit_factor(df['pl_ratio'].iloc[-1])
        stats['Accuracy'] = self.accuracy(df['pl_ratio'].iloc[-1])
        # stats['Log Loss']=logLoss(daily_return)
        self.__stats = stats

    def annualized_return(self, total_return, total_days):
        annualized_return = ((1 + total_return) **
                             (252.0 / np.float(total_days)) - 1)

        return annualized_return

    def annualized_std(self, variance, startingCapital):
        return (np.sqrt(252) * np.sqrt(variance)) / float(startingCapital)

    def annualized_downside_std(self, daily_return):
        downside_return = daily_return.copy()
        downside_return[downside_return > 0] = 0
        return np.sqrt(252) * np.std(downside_return)

    def annual_vol(self, variance, startingCapital):
        return self.annualized_std(variance, startingCapital)

    def sharpe_ratio(self, annual_return, annual_vol):
        if annual_vol == 0:
            return np.nan
        else:
            return annual_return / float(annual_vol)

    def sortino_ratio(self, total_return, total_days, daily_return):
        stdev = self.annualized_downside_std(daily_return)
        if stdev == 0:
            return np.nan
        else:
            return self.annualized_return(total_return, total_days) / stdev

    def max_drawdown(self, maxDrawdown, startingCapital):
        return maxDrawdown['maxDrawdown'] / float(startingCapital)
        # return np.max(np.maximum.accumulate(portfolioValue) - portfolioValue) / portfolioValue[0]

    def roc(self, total_pnl, capitalUsage):
        if capitalUsage > 0:
            return total_pnl / float(capitalUsage)
        else:
            return np.nan

    def beta(self, daily_return, baseline_daily_return):
        stdev = np.std(baseline_daily_return)
        if stdev == 0:
            return np.nan
        else:
            return np.corrcoef(daily_return, baseline_daily_return)[0, 1] * np.std(daily_return) / stdev

    def alpha(self, daily_return, baseline_daily_return, beta):
        return self.annualized_return(daily_return) - beta * self.annualized_return(baseline_daily_return)

    def profit_factor(self, plRatioDict):
        if plRatioDict['totalLoss'] == 0:
            return float('nan')
        return plRatioDict['totalProfit'] / float(plRatioDict['totalLoss'])

    def profitability(self, plRatioDict, total_pnl):
        if total_pnl == 0:
            return 0
        return plRatioDict['totalProfit'] / float(total_pnl)

    def profit_factor_avg(self, plRatioDict):
        if plRatioDict['totalLoss'] == 0:
            return float('nan')
        return (plRatioDict['totalProfit'] / float(plRatioDict['countProfit']))\
            / (plRatioDict['totalLoss'] / float(plRatioDict['countLoss']))

    def accuracy(self, plRatioDict):
        total_count = (plRatioDict['countProfit'] + plRatioDict['countLoss'])
        if total_count == 0:
            return 0
        return plRatioDict['countProfit'] / float(total_count)

    def getBenchmarkData(self, baseSymbol, priceFeature, folderName):
        if (baseSymbol is None):
            return None

        baseline_data = {}
        path = folderName + '/' + baseSymbol + '_features.csv'
        csv = pd.read_csv(path, engine='python')
        csv.set_index(csv['time'], inplace=True)
        csv.index = pd.to_datetime(csv.index)
        # features = [col.upper() for col in csv.columns]
        baseline_data['price'] = self.resampleData(
            csv[priceFeature], '1D').last()
        start = baseline_data['price'][0]
        baseline_data['returns'] = baseline_data['price'] / start - 1
        baseline_data['total_return'] = baseline_data['returns'][len(
            baseline_data['price']) - 1]
        baseline_data['daily_returns'] = baseline_data['price'] / \
            baseline_data['price'].shift(1) - 1
        baseline_data['daily_returns'].dropna(inplace=True)
        return baseline_data



class InstrumentExection:
    def __init__(self, time, instrumentId, volumne, executionType):
        self.__timeOfExecution = time
        self.__instrumentId = instrumentId
        self.__volume = volumne
        self.__executionType = executionType

    def getTimeOfExecution(self):
        return self.__timeOfExecution

    def getInstrumentId(self):
        return self.__instrumentId

    def getVolume(self):
        return self.__volume

    def getExecutionType(self):
        return self.__executionType


class BaseExecutionSystem(object):

    '''
    Returns an array of InstrumentExecutions
    '''
    def getExecutions(self, time, instrumentsManager, capital):
        return []



from datetime import datetime
from backtester.instrumentUpdates import *
from backtester.constants import *
from backtester.logger import *
from backtester.dataSource.data_source import DataSource
import os
import os.path
from backtester.dataSource.data_source_utils import downloadFileFromYahoo, groupAndSortByTimeUpdates

TYPE_LINE_UNDEFINED = 0
TYPE_LINE_HEADER = 1
TYPE_LINE_DATA = 2


def checkDate(lineItem):
    try:
        datetime.strptime(lineItem, '%Y-%m-%d')
        return True
    except ValueError:
        return False


def checkTimestamp(lineItem):
    return True


def isFloat(string):
    try:
        return float(string) or float(string) == 0.0
    except ValueError:  # if string is not a number
        return False

# Returns the type of lineItems


def validateLineItem(lineItems):
    if len(lineItems) == 7:
        if lineItems[0] == "Date":
            return TYPE_LINE_HEADER
        elif checkDate(lineItems[0]) and isFloat(lineItems[1]) and isFloat(lineItems[2]) and isFloat(lineItems[3]) and isFloat(lineItems[4]) and isFloat(lineItems[5]):
            return TYPE_LINE_DATA
    return TYPE_LINE_UNDEFINED


def parseDataLine(lineItems):
    if (len(lineItems) != 7):
        return None
    openPrice = float(lineItems[1])
    high = float(lineItems[2])
    low = float(lineItems[3])
    closePrice = float(lineItems[4])
    adjustedClose = float(lineItems[5])
    volume = float(lineItems[6])
    return {'open': openPrice,
            'high': high,
            'low': low,
            'close': closePrice,
            'adjClose' : adjustedClose,
            'volume': volume}


class InstrumentsFromFile():
    def __init__(self, fileName, instrumentId):
        self.fileName = fileName
        self.instrumentId = instrumentId
        self.currentInstrumentSymbol = instrumentId
        self.currentTimeOfUpdate = None
        self.currentBookData = None

    def processLine(self, line):
        lineItems = line.split(',')
        lineItemType = validateLineItem(lineItems)
        if (lineItemType == TYPE_LINE_DATA):
            inst = None
            if self.currentInstrumentSymbol is not None:
                self.currentTimeOfUpdate = datetime.strptime(lineItems[0], "%Y-%m-%d")
                self.currentInstrumentSymbol = self.instrumentId
                self.currentBookData = parseDataLine(lineItems)
                if self.currentBookData is None:
                    return None
                # right now only works for stocks
                inst = StockInstrumentUpdate(stockInstrumentId=self.instrumentId,
                                             tradeSymbol=self.currentInstrumentSymbol,
                                             timeOfUpdate=self.currentTimeOfUpdate,
                                             bookData=self.currentBookData)
                return inst
        return None

    def processLinesIntoInstruments(self):
        with open(self.fileName, "r") as ins:
            instruments = []
            for line in ins:
                inst = self.processLine(line)
                if inst is not None:
                    instruments.append(inst)
            return instruments


class YahooStockDataSource(DataSource):
    def __init__(self, cachedFolderName, instrumentIds, startDateStr, endDateStr,event='history'):
        self.startDate = datetime.strptime(startDateStr, "%Y/%m/%d")
        self.endDate = datetime.strptime(endDateStr, "%Y/%m/%d")
        self.cachedFolderName = cachedFolderName
        self.instrumentIds = instrumentIds
        self.currentDate = self.startDate
        self.event = event

    def getFileName(self, instrumentType, instrumentId):
        return '%s/%s_%s_%s_%s.csv' % (self.cachedFolderName, instrumentId, instrumentType, self.startDate.strftime("%Y%m%d"), self.endDate.strftime("%Y%m%d"))

    def emitInstrumentUpdates(self):
        allInstrumentUpdates = []
        for instrumentId in self.instrumentIds:
            fileName = self.getFileName(INSTRUMENT_TYPE_STOCK, instrumentId)
            if not os.path.exists(self.cachedFolderName):
                os.mkdir(self.cachedFolderName, 0o755)
            if not os.path.isfile(fileName):
                downloadFileFromYahoo(self.startDate, self.endDate,instrumentId, fileName, event='history')
            fileHandler = InstrumentsFromFile(fileName=fileName, instrumentId=instrumentId)
            instrumentUpdates = fileHandler.processLinesIntoInstruments()
            allInstrumentUpdates = allInstrumentUpdates + instrumentUpdates
        groupedInstrumentUpdates = groupAndSortByTimeUpdates(allInstrumentUpdates)
        for timeOfUpdate, instrumentUpdates in groupedInstrumentUpdates:
            yield([timeOfUpdate, instrumentUpdates])


if __name__ == "__main__":
    instrumentIds = ['IBM', 'AAPL', 'MSFT']
    startDateStr = '2017/05/10'
    endDateStr = '2017/06/09'
    YahooStockDataSource(cachedFolderName='yahooData',
                                     instrumentIds=instrumentIds,
                                     startDateStr=startDateStr,
                                     endDateStr=endDateStr,
                                     event='history')



from backtester.features.feature import Feature
import numpy as np


class DirectionFeature(Feature):

    @classmethod
    def computeForLookbackData(cls, featureParams, featureKey, currentFeatures, lookbackDataDf):
        data = lookbackDataDf[featureParams['featureName']]
        if len(data.index) < featureParams['period']:
            return 0
        return np.sign(currentFeatures[featureParams['featureName']] - data[-featureParams['period']])



from backtester.executionSystem.simple_execution_system_fairvalue import SimpleExecutionSystemWithFairValue
from backtester.logger import *
import numpy as np


class QQExecutionSystem(SimpleExecutionSystemWithFairValue):
    def __init__(self, basisEnter_threshold=0.1, basisExit_threshold=0.05,
                 basisLongLimit=5000, basisShortLimit=5000,
                 basisCapitalUsageLimit=0.05, basisLotSize=100,
                 basisLimitType='L', basis_thresholdParam='sdev',
                 price='', feeDict=0.05):
        super(QQExecutionSystem, self).__init__(enter_threshold_deviation=basisEnter_threshold,
                                                exit_threshold_deviation=basisExit_threshold,
                                                longLimit=basisLongLimit, shortLimit=basisShortLimit,
                                                capitalUsageLimit=basisCapitalUsageLimit, lotSize=basisLotSize,
                                                limitType=basisLimitType, price=price)
        self.fees = feeDict
        self.thresholdParam = basis_thresholdParam

    def getBuySell(self, instrument, currentPredictions):
        instrumentId = instrument.getInstrumentId()
        try:
            currentPrice = instrument.getDataDf()[self.priceFeature].iloc[-1]
            fairValue = currentPredictions[instrumentId]
            currentDeviationFromPrediction = currentPrice - fairValue
            return -np.sign(currentDeviationFromPrediction)
        except KeyError:
            logError('You have specified FairValue Execution Type but Price Feature Key does not exist')

    def enterCondition(self, instrumentsManager, instrument, currentPredictions):
        instrumentId = instrument.getInstrumentId()
        try:
            currentPrice = instrument.getDataDf()[self.priceFeature].iloc[-1]
            fairValue = currentPredictions[instrumentId]
            currentDeviationFromPrediction = currentPrice - fairValue
            return np.abs(currentDeviationFromPrediction) -\
                2 * self.fees > (self.enter_threshold) * np.abs(instrument.getDataDf()[self.thresholdParam].iloc[-1])
        except KeyError:
            logError('You have specified FairValue Execution Type but Price Feature Key does not exist')

    def exitCondition(self, instrumentsManager, instrument, currentPredictions):
        instrumentId = instrument.getInstrumentId()
        try:
            currentPrice = instrument.getDataDf()[self.priceFeature].iloc[-1]
            fairValue = currentPredictions[instrumentId]
            currentDeviationFromPrediction = currentPrice - fairValue
            position = instrument.getCurrentPosition()
            return -np.sign(position)*(currentDeviationFromPrediction) < (self.exit_threshold) * np.abs(instrument.getDataDf()[self.thresholdParam].iloc[-1])
        except KeyError:
            logError('You have specified FairValue Execution Type but Price Feature Key does not exist')

    def hackCondition(self, instrumentsManager):
        return False

    # def getExecutions(self, time, instrumentsManager, capital):
    #   # TODO:
    #   marketFeaturesDf = instrumentsManager.getDataDf()
    #   currentMarketFeatures = marketFeaturesDf.iloc[-1]
    #   currentPredictions = marketFeaturesDf['prediction'].iloc[-1]
    #   logInfo(str(currentPredictions))
    #   currentDeviationFromPredictions = {}
    #   currentProbabilityPredictions = {}
    #   for instrumentId in currentPredictions.keys():
    #       instrument = instrumentsManager.getInstrument(instrumentId)
    #       if instrument is None:
    #           continue
    #       try:
    #           currentPrice = instrument.getDataDf()[self.priceFeature].iloc[-1]
    #       except KeyError:
    #           logError('You have specified FairValue Execution Type but Price Feature Key does not exist')

    #       currentDeviationFromPrediction = currentPredictions[instrumentId]/currentPrice
    #       currentProbabilityPredictions[instrumentId] = currentDeviationFromPrediction/2

    #   executions = []
    #   executions += self.exitPosition(instrumentsManager, currentProbabilityPredictions)
    #   executions += self.enterPosition(instrumentsManager, currentProbabilityPredictions, capital)
    #   return executions



from backtester.constants import *
'''
A base class to represent an update to instrument.
'''


class InstrumentUpdate(object):
    '''
    instrumentId: unique id to represent the instrument
    timeOfUpdate: datetime object for time of update
    expiryTime: datettime object for expiry of option
    bookData: dictionary of things specific to the instrument.
    '''
    def __init__(self, instrumentId, tradeSymbol, timeOfUpdate, bookData):
        self.__instrumentId = instrumentId
        self.__tradeSymbol = tradeSymbol
        self.__timeOfUpdate = timeOfUpdate
        self.__bookData = bookData

    '''
    Returns a string to represent a unique identifier for the instrument
    '''
    def getInstrumentId(self):
        return self.__instrumentId

    def getTradeSymbol(self):
        return self.__tradeSymbol

    def getBookData(self):
        return self.__bookData

    def getBookDataFeatures(self):
        return self.__bookData.keys()

    def getTimeOfUpdate(self):
        return self.__timeOfUpdate

    def getTypeOfInstrument(self):
        raise "Unimplemented error"
        return INSTRUMENT_TYPE_UNDEFINED



from backtester.features.feature import Feature


class ProfitLossFeature(Feature):

    '''
    Computing for Instrument. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForInstrument(cls, featureParams, featureKey, currentFeatures, instrument, instrumentManager):

        priceDict = instrument.getDataDf()[featureParams['price']]
        pnlDict = instrument.getDataDf()[featureKey]
        if len(priceDict) < 1:
            return 0
        cumulativePnl = 0 if (len(pnlDict) <= 1) else pnlDict.values[-2]
        fees = currentFeatures[featureParams['fees']]
        currentPosition = instrument.getCurrentPosition()
        previousPosition = instrument.getDataDf()['position'][-2] if (len(instrument.getDataDf()['position']) > 1) else 0
        previousPrice = priceDict[-2] if (len(priceDict) > 1) else 0
        currentPrice = currentFeatures[featureParams['price']]
        changeInPosition = currentPosition - previousPosition
        tradePrice = instrument.getLastTradePrice()
        pnl = (previousPosition * (currentPrice - previousPrice)) + (changeInPosition * (currentPrice - tradePrice)) - fees
        cumulativePnl += pnl
        return cumulativePnl

    '''
    Computing for Market. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForMarket(cls, featureParams, featureKey, currentMarketFeatures, instrumentManager):
        pnl = 0
        pnlDict = instrumentManager.getDataDf()[featureKey]
        pnlKey = 'pnl'
        if 'instrument_pnl_feature' in featureParams:
            pnlKey = featureParams['instrument_pnl_feature']
        if len(pnlDict) < 1:
            return 0
        allInstruments = instrumentManager.getAllInstrumentsByInstrumentId()
        for instrumentId in allInstruments:
            instrument = allInstruments[instrumentId]
            pnl += instrument.getDataDf()[pnlKey][-1]
        return pnl



import pandas as pd
import numpy as np
import plotly
from plotly.graph_objs import Scatter, Layout
from os import listdir
from os.path import isfile, join, basename
from backtester.metrics.metrics import Metrics
from backtester.logger import *

'''
Usage(to test in console):
>> pip install plotly
>> python
>> from plotter import plot
>> plot(r'./runLogs/runLog_20170613_201741', []) # plots every file in directory
>> plot(r'./runLogs/runLog_20170613_201741', ['ADBE_features.csv']) # ignores the files listed here
>> plot(r'./runLogs/runLog_20170613_201741/ADBE_features.csv', []) # plots only ADBE_features
Replace the appropriate timeStamps

TODO: 1) Support excluding columns for each files.
      2) Cleanup the html files generated from plotting/dont regenerate and recycle ones present.
      3) Provide a selector GUI to chose the files.
'''

def generateGraph(df, fileName, stats, benchmark_pnl):
    layout = dict(
        title=stats,
        xaxis=dict(
            rangeselector=dict(
                buttons=list([
                    dict(count=6,
                         label='6m',
                         step='month',
                         stepmode='backward'),
                    dict(count=1,
                         label='YTD',
                         step='year',
                         stepmode='todate'),
                    dict(count=1,
                         label='1y',
                         step='year',
                         stepmode='backward'),
                    dict(count=5,
                         label='5y',
                         step='year',
                         stepmode='backward'),
                    dict(step='all')
                ])
            ),
            rangeslider=dict(),
            type='date'
        )
    )
    plot_data = {
        "data": [],
        "layout": layout
    }
    for col in df.columns[1:]:
        plot_data['data'] += [Scatter(x=df.index, y=df[col], name=col)]
    if benchmark_pnl is not None:
        plot_data['data'] += [Scatter(x=df.index,
                                      y=100 * benchmark_pnl, name='Benchmark (%)')]
    plotly.offline.plot(plot_data, filename=fileName + ".html")



from backtester.features.feature import Feature


class MaxDrawdownFeature(Feature):

    '''
    Computing for Instrument. By default defers to computeForLookbackData
    '''

    '''MAKE SURE PNL and PORTFOLIO VALUEis calculated BEFORE this feature
    '''
    @classmethod
    def computeForInstrument(cls, featureParams, featureKey, currentFeatures, instrument, instrumentManager):
        raise NotImplemented

        return None

    '''
    Computing for Market. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForMarket(cls, featureParams, featureKey, currentMarketFeatures, instrumentManager):
        portfolioValueKey = 'portfolio_value'
        lookbackMarketDataDf = instrumentManager.getDataDf()
        if len(lookbackMarketDataDf) < 1 or instrumentManager is None:
            # First Iteration
            return 0
        if 'portfolioValueKey' in featureParams:
            portfolioValueKey = featureParams['portfolioValueKey']

        portfolioValueDict = lookbackMarketDataDf[portfolioValueKey]
        if len(portfolioValueDict) <= 1:
            return {'maxPortfolioValue': 0, 'maxDrawdown': 0}
        drawdownDict = lookbackMarketDataDf[featureKey].iloc[-2]

        maxPortfolioValue = portfolioValueDict.iloc[-1] if drawdownDict['maxPortfolioValue'] < portfolioValueDict.iloc[-1] \
            else drawdownDict['maxPortfolioValue']

        maxDrawdown = maxPortfolioValue - portfolioValueDict.iloc[-1] if drawdownDict['maxDrawdown'] < maxPortfolioValue - portfolioValueDict.iloc[-1] \
            else drawdownDict['maxDrawdown']

        return {'maxPortfolioValue': maxPortfolioValue, 'maxDrawdown': maxDrawdown}



from backtester.features.feature import Feature
import numpy as np


class ScoreLogLossFeature(Feature):

    @classmethod
    def computeForInstrument(cls, featureParams, featureKey, currentFeatures, instrument, instrumentManager):
        lookbackDataDf = instrument.getDataDf()
        predictionKey = 'prediction'
        target = 'y'
        countKey = 'count'
        if len(lookbackDataDf) < 1 or instrumentManager is None:
            return 0
        lookbackMarketDataDf = instrumentManager.getDataDf()
        if 'predictionKey' in featureParams:
            predictionKey = featureParams['predictionKey']
        if 'target' in featureParams:
            target = featureParams['target']
        if 'countKey' in featureParams:
            countKey = featureParams['countKey']
        if len(lookbackMarketDataDf) < 1:
            # first iteration
            return 0
        predictionDict = lookbackMarketDataDf[predictionKey].iloc[-1]
        prevCount = lookbackMarketDataDf[countKey].iloc[-1]
        if len(predictionDict) == 0 or prevCount == 0:
            return 0
        prevData = lookbackDataDf[featureKey].iloc[-1]
        temp = (prevCount - 1) * prevData

        p = predictionDict[instrument.getInstrumentId()]
        if np.isnan(p):
            p = 0.5
        y = (1 + lookbackDataDf[target].iloc[-1]) / 2
        temp = temp - (np.log(p) * float(y) + np.log(1 - p) * float(1 - y))

        return float(temp) / float(prevCount)

    '''
    Computing for Market. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForMarket(cls, featureParams, featureKey, currentMarketFeatures, instrumentManager):
        score = 0
        scoreDict = instrumentManager.getDataDf()[featureKey]
        scoreKey = 'score'
        if 'instrument_score_feature' in featureParams:
            scoreKey = featureParams['instrument_score_feature']
        if len(scoreDict) < 1:
            return 0
        cumulativeScore = scoreDict.values[-1]
        allInstruments = instrumentManager.getAllInstrumentsByInstrumentId()
        for instrumentId in allInstruments:
            instrument = allInstruments[instrumentId]
            lookbackDataDf = instrument.getDataDf()
            score += instrument.getDataDf()[scoreKey][-1]
        cumulativeScore += score / len(allInstruments)
        return score



from __future__ import absolute_import, division, print_function, unicode_literals
import numpy as np
import pandas as pd
import math


def writecsv(results, filename):
    print('writing %s.csv' % filename)
    try:
        csv_file = open('%s.csv' % filename, 'wb')
        results.to_csv(csv_file)
    except:
        csv_file = open('%s.csv' % filename, 'w')
        results.to_csv(csv_file)
    csv_file.close()


def calc_realized(last, fut, period, time):
    if last != 0:
        fut = pd.concat([pd.DataFrame([last]), fut])
    ret = fut / fut.shift(1) - 1
    ret.dropna(inplace=True)
    sqsum = (ret**2).rolling(window=period).sum()
    rvol = np.sqrt(252 * sqsum / time)
    return rvol


def calc_var_RT(var, fut, fut0):
    if fut0 > 0:
        var = var + (fut / fut0 - 1)**2
    return var


def ema_RT(prev_ema, data, period):
    alpha = 1 - math.exp(math.log(0.5) / period)
    avg = data * alpha + prev_ema * (1 - alpha)
    return avg


def ma(data, period):
    avg = data.rolling(window=period).mean()
    return avg


def ema(data, period):
    avg = data.ewm(halflife=period).mean()
    return avg

def msdev(data,period):
    s = data.rolling(window=period).std()
    return s
    
def convert_time(timestamp):
    try:
        d = pd.to_datetime(timestamp, infer_datetime_format=True)
    except:
        if isinstance(timestamp, basestring):
            d = dt.datetime.strptime(timestamp, "%Y/%m/%d %H:%M:%S:%f")
        elif type(timestamp) == float or type(timestamp) == long or type(timestamp) == np.float64:
            (year, month, day, hour, minute, second, microsecond) = (str(timestamp)[0:4], str(timestamp)[4:6], str(
                timestamp)[6:8], str(timestamp)[8:10], str(timestamp)[10:12], str(timestamp)[12:14], str(timestamp)[14:])
            d = dt(int(year), int(month), int(day), int(hour), int(minute), int(second), int(microsecond))
        else:
            d = timestamp
    return d


def calculate_t(eval_date, exp_date):
    t = calculate_t_days(eval_date, exp_date) / 365.0
    return t


def calculate_t_days(eval_date, exp_date):
    d0 = convert_time(eval_date)
    d1 = convert_time(exp_date)
    sec = (d1 - d0).seconds
    if(sec < 3600):
        sec = (sec) / 12000.0
    elif(sec < 21300):
        sec = sec / 70800.0 + 0.25
    elif (sec < 22500):
        sec = sec / 12000.0 - 1.225
    else:
        sec = 1
    t = (busday_count(d0, d1) + sec)
    return t


def busday_count(start, end):
    daydiff = end.weekday() - start.weekday()
    days = ((end - start).days - daydiff) / 7 * 5 + min(daydiff, 5) - (max(end.weekday() - 4, 0) % 5)
    return days



import csv
import os
import sys


class StateWriter:

    def __init__(self, parentFolderName, runName):
        self.__runName = runName
        if not os.path.exists(parentFolderName):
            os.mkdir(parentFolderName, 0o755)
        self.__folderName = parentFolderName + '/' + 'runLog_' + runName
        if not os.path.exists(self.__folderName):
            os.mkdir(self.__folderName, 0o755)
        self.__openFiles = []
        self.__marketFeaturesFilename = None
        self.__marketFeaturesWriter = None
        self.__instrumentIdToWriters = {}

    def getMarketFeaturesFilename(self):
        return self.__marketFeaturesFilename

    def getFolderName(self):
        return self.__folderName
        
    def closeStateWriter(self):
        for file in self.__openFiles:
            file.close()

    def writeColumns(self, writer, df):
        featureKeys = list(df.columns)
        toSaveColumns = ['time'] + featureKeys
        writer.writerow(toSaveColumns)

    def writeLastFeatures(self, writer, df):
        if len(df) == 0:
            return
        lastFeatures = df.iloc[-1]
        timeOfUpdate = lastFeatures.name
        featureValues = lastFeatures.values
        toSaveRow = [timeOfUpdate] + list(featureValues)
        writer.writerow(toSaveRow)

    def writeCurrentState(self, instrumentManager):
        marketFeaturesDf = instrumentManager.getDataDf()
        if self.__marketFeaturesWriter is None:
            self.__marketFeaturesFilename = self.__folderName + '/marketFeatures.csv'
            if sys.version_info >= (3,):
                marketFeaturesFile =  open(self.__marketFeaturesFilename, 'w', encoding='utf8', newline='')
            else:
                marketFeaturesFile =  open(self.__marketFeaturesFilename, 'wb')
            self.__openFiles.append(marketFeaturesFile)
            self.__marketFeaturesWriter = csv.writer(marketFeaturesFile)
            self.writeColumns(self.__marketFeaturesWriter, marketFeaturesDf)
        self.writeLastFeatures(self.__marketFeaturesWriter, marketFeaturesDf)
        instrumentsDict = instrumentManager.getAllInstrumentsByInstrumentId()
        for instrumentId in instrumentsDict:
            instrument = instrumentsDict[instrumentId]
            instrumentFeaturesDf = instrument.getDataDf()
            if instrumentId not in self.__instrumentIdToWriters:
                instrumentFeaturesFilename = self.__folderName + '/' + instrumentId + '_features.csv'
                if sys.version_info >= (3,):
                    instrumentFeaturesFile =  open(instrumentFeaturesFilename, 'w', encoding='utf8', newline='')
                else:
                    instrumentFeaturesFile =  open(instrumentFeaturesFilename, 'wb')
                self.__openFiles.append(instrumentFeaturesFile) 
                self.__instrumentIdToWriters[instrumentId] = csv.writer(instrumentFeaturesFile)
                self.writeColumns(self.__instrumentIdToWriters[instrumentId], instrumentFeaturesDf)
            instrumentFeaturesWriter = self.__instrumentIdToWriters[instrumentId]
            self.writeLastFeatures(instrumentFeaturesWriter, instrumentFeaturesDf)



from backtester.executionSystem.simple_execution_system import SimpleExecutionSystem


class PairExecutionSystem(SimpleExecutionSystem):
    def __init__(self, pair, pairRatio, pairEnter_threshold=0.7, pairExit_threshold=0.55, pairLongLimit=10, pairShortLimit=10, pairCapitalUsageLimit=0, pairLotSize=1):
        longLimit = {pair[0]: pairLongLimit, pair[1]: pairLongLimit * pairRatio}
        shortLimit = {pair[0]: pairShortLimit, pair[1]: pairShortLimit * pairRatio}
        lotSize = {pair[0]: pairLotSize, pair[1]: pairLotSize * pairRatio}
        super(PairExecutionSystem, self).__init__(enter_threshold=pairEnter_threshold,
                                                  exit_threshold=pairExit_threshold,
                                                  longLimit=longLimit,
                                                  shortLimit=shortLimit,
                                                  capitalUsageLimit=pairCapitalUsageLimit,
                                                  lotSize=lotSize)



from backtester.features.feature import Feature
from backtester.financial_fn import ma


class MACDFeature(Feature):

    @classmethod
    def computeForLookbackData(cls, featureParams, featureKey, currentFeatures, lookbackDataDf):
        data = lookbackDataDf[featureParams['featureName']]
        avg1 = data[-featureParams['period1']:].mean()
        avg2 = data[-featureParams['period2']:].mean()
        if len(data) < 1:
        	return 0
        return (avg1 - avg2)



from backtester.features.feature import Feature
from backtester.financial_fn import ma


class RSIFeature(Feature):

    @classmethod
    def computeForLookbackData(cls, featureParams, featureKey, currentFeatures, lookbackDataDf):
        data = lookbackDataDf[featureParams['featureName']]
        data_upside = data.sub(data.shift(1), fill_value=0)
        data_downside = data_upside.copy()
        data_downside[data_upside > 0] = 0
        data_upside[data_upside < 0] = 0
        if len(data.index) > 0:
            avg_upside = ma(data_upside, featureParams['period'])[-1]
            avg_downside = - ma(data_downside, featureParams['period'])[-1]
            rs = avg_upside / avg_downside
        else:
            return 0 

        return 100 - (100 * avg_downside / (avg_downside + avg_upside ))



from backtester.features.feature import Feature


class PositionInstrumentFeature(Feature):

    '''
    Computing for Instrument. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForInstrument(cls, featureParams, featureKey, currentFeatures, instrument, instrumentManager):
        return instrument.getCurrentPosition()

    '''
    Computing for Market. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForMarket(cls, featureParams, featureKey, currentMarketFeatures, instrumentManager):
        raise NotImplementedError



from backtester.instrumentUpdates.instrument_update import InstrumentUpdate
from backtester.constants import *


class FutureInstrumentUpdate(InstrumentUpdate):
    '''
    futureInstrumentId: symbol for future id
    timeOfUpdate: datetime object for time of update
    expiryTime: datettime object for expiry of future
    bookData: dictionary. keys which we support right now:
             bidPrice
             bidVolume
             askPrice
             askVolume
             volume
             openInterest
             open
             close
             high
             low
    '''
    def __init__(self, futureInstrumentId, tradeSymbol, timeOfUpdate, bookData, expiryTime, underlyingInstrumentId):
        super(FutureInstrumentUpdate, self).__init__(futureInstrumentId, tradeSymbol, timeOfUpdate, bookData)
        self.__futureInstrumentId = futureInstrumentId
        self.__expiryTime = expiryTime
        self.__underlyingInstrumentId = underlyingInstrumentId

    def getFutureInstrumentId(self):
        return self.__futureInstrumentId

    def getTypeOfInstrument(self):
        return INSTRUMENT_TYPE_FUTURE

    def getExpiryTime(self):
        return self.__expiryTime

    def getUnderlyingInstrumentId(self):
        return self.__underlyingInstrumentId



from backtester.features.feature import Feature

class ProfitabilityFeature(Feature):

    @classmethod
    def computeForLookbackData(cls, featureParams, featureKey, currentFeatures, lookbackDataDf):
        pnlKey = 'pnl'
        countKey = 'count'
        if 'pnlKey' in featureParams:
            pnlKey = featureParams['pnlKey']
        if 'countKey' in featureParams:
            countKey = featureParams['countKey']
        prevData = lookbackDataDf[featureKey].iloc[-1]
        prevCount = lookbackDataDf[countKey].iloc[-1]
        accurateCount = prevCount*prevData
        if(lookbackDataDf[pnlKey].iloc[-1] > 0):
            accurateCount += 1
        prevCount += 1
        return float(accurateCount)/float(prevCount)



from backtester.features.feature import Feature
from backtester.financial_fn import ma
import numpy as np


class MovingMaximumFeature(Feature):

    @classmethod
    def computeForLookbackData(cls, featureParams, featureKey, currentFeatures, lookbackDataDf):
        data = lookbackDataDf[featureParams['featureName']]
        #avg = ma(data, featureParams['period'])
        return data[-featureParams['period']:].max()



from backtester.features.feature import Feature
from backtester.financial_fn import ma
import numpy as np
from scipy import stats
from scipy import spatial

class MARibbonDistanceFeature(Feature):

    @classmethod
    def computeForLookbackData(cls, featureParams, featureKey, currentFeatures, lookbackDataDf):
        data = lookbackDataDf[featureParams['featureName']]
        rolling_means = np.zeros(featureParams['numRibbons'])
        if len(data) < 1:
        	return rolling_means
        space = int((featureParams['endPeriod'] - featureParams['startPeriod'])/(featureParams['numRibbons']-1))
        for idx in np.linspace(featureParams['startPeriod'], featureParams['endPeriod'], featureParams['numRibbons']):
            i = int(idx)
            rolling_means[i/space - 1] = data[-i:].mean()
        ranking = stats.rankdata(rolling_means)
        d = spatial.distance.hamming(ranking, range(1, 1+len(rolling_means)))
        return d



from datetime import datetime
from backtester.instrumentUpdates import *
from backtester.constants import *
from backtester.logger import *
from backtester.dataSource.data_source import DataSource
import os
import os.path
from pandas_datareader import data
from backtester.dataSource.data_source_utils import downloadFileFromYahoo, groupAndSortByTimeUpdates

TYPE_LINE_UNDEFINED = 0
TYPE_LINE_HEADER = 1
TYPE_LINE_DATA = 2


def checkDate(lineItem):
    try:
        datetime.strptime(lineItem, '%Y-%m-%d')
        return True
    except ValueError:
        return False


def checkTimestamp(lineItem):
    return True


def isFloat(string):
    try:
        return float(string) or float(string) == 0.0
    except ValueError:  # if string is not a number
        return False

# Returns the type of lineItems


def validateLineItem(lineItems, lineLength):
    if len(lineItems) == lineLength:
        if lineItems[0] == "Date":
            return TYPE_LINE_HEADER
        elif checkDate(lineItems[0]) and isFloat(lineItems[1]) and isFloat(lineItems[2]) and isFloat(lineItems[3]) and isFloat(lineItems[4]) and isFloat(lineItems[5]):
            return TYPE_LINE_DATA
    return TYPE_LINE_UNDEFINED


def parseDataLine(lineItems, lineLength):
    if (len(lineItems) != lineLength):
        return None
    openPrice = float(lineItems[1])
    high = float(lineItems[2])
    low = float(lineItems[3])
    closePrice = float(lineItems[4])
    volume = float(lineItems[5])
    return {'open': openPrice,
            'high': high,
            'low': low,
            'close': closePrice,
            'volume': volume}


class InstrumentsFromFile():
    def __init__(self, fileName, instrumentId):
        self.fileName = fileName
        self.instrumentId = instrumentId
        self.currentInstrumentSymbol = instrumentId
        self.currentTimeOfUpdate = None
        self.currentBookData = None

    def processLine(self, line, lineLength):
        lineItems = line.split(',')
        lineItemType = validateLineItem(lineItems, lineLength)
        if (lineItemType == TYPE_LINE_DATA):
            inst = None
            if self.currentInstrumentSymbol is not None:
                self.currentTimeOfUpdate = datetime.strptime(lineItems[0], "%Y-%m-%d")
                self.currentInstrumentSymbol = self.instrumentId
                self.currentBookData = parseDataLine(lineItems, lineLength)
                if self.currentBookData is None:
                    return None
                # right now only works for stocks
                inst = StockInstrumentUpdate(stockInstrumentId=self.instrumentId,
                                             tradeSymbol=self.currentInstrumentSymbol,
                                             timeOfUpdate=self.currentTimeOfUpdate,
                                             bookData=self.currentBookData)
                return inst
        return None

    def processLinesIntoInstruments(self, lineLength):
        with open(self.fileName, "r") as ins:
            instruments = []
            for line in ins:
                inst = self.processLine(line, lineLength)
                if inst is not None:
                    instruments.append(inst)
            return instruments


class GoogleStockDataSource(DataSource):
    def __init__(self, cachedFolderName, instrumentIds, startDateStr, endDateStr):
        self.startDate = datetime.strptime(startDateStr, "%Y/%m/%d")
        self.endDate = datetime.strptime(endDateStr, "%Y/%m/%d")
        self.cachedFolderName = cachedFolderName
        self.instrumentIds = instrumentIds
        self.currentDate = self.startDate
        self.lineLength = 6

    def downloadFile(self, instrumentId, fileName):
        logInfo('Downloading %s' % fileName)
        pd = data.DataReader(instrumentId, 'google', self.startDate, self.endDate)
        pd.to_csv(fileName)

    def getFileName(self, instrumentType, instrumentId):
        return '%s/%s_%s_%s_%s.csv' % (self.cachedFolderName, instrumentId, instrumentType, self.startDate.strftime("%Y%m%d"), self.endDate.strftime("%Y%m%d"))

    def emitInstrumentUpdates(self, adjustPrice=True):
        allInstrumentUpdates = []

        for instrumentId in self.instrumentIds:
            fileName = self.getFileName(INSTRUMENT_TYPE_STOCK, instrumentId)
            if not os.path.exists(self.cachedFolderName):
                os.mkdir(self.cachedFolderName, 0o755)
            if not os.path.isfile(fileName):
                self.downloadFile(instrumentId, fileName)
                if adjustPrice:
                    self.adjustPriceForSplitAndDiv(instrumentId, fileName)
            fileHandler = InstrumentsFromFile(fileName=fileName, instrumentId=instrumentId)
            instrumentUpdates = fileHandler.processLinesIntoInstruments()
            allInstrumentUpdates = allInstrumentUpdates + instrumentUpdates

        groupedInstrumentUpdates = groupAndSortByTimeUpdates(allInstrumentUpdates)
        for timeOfUpdate, instrumentUpdates in groupedInstrumentUpdates:
            yield([timeOfUpdate, instrumentUpdates])

    def adjustPriceForSplitAndDiv(self, instrumentId, fileName):
        divFile = self.getFileName('div', instrumentId)
        splitFile = self.getFileName('split', instrumentId)
        if not (os.path.isfile(divFile) and os.path.isfile(splitFile)):
            downloadFileFromYahoo(self.startDate, self.endDate, '%s.NS' % instrumentId, divFile, event='div')
            downloadFileFromYahoo(self.startDate, self.endDate, '%s.NS' % instrumentId, splitFile, event='split')
        div = pd.read_csv(divFile, engine='python', index_col='Date', parse_dates=True)
        split = pd.read_csv(splitFile, engine='python', index_col='Date', parse_dates=True)
        prices = pd.read_csv(fileName, engine='python', index_col='Date', parse_dates=True)
        temp = pd.concat([div, prices], axis=1).fillna(0)
        interim = (temp['Close'] - temp['Dividends']) / temp['Close']
        multiplier1 = interim.sort_index(ascending=False).cumprod().sort_index(ascending=True)
        temp2 = split['Stock Splits'].str.split('/', expand=True)
        if len(temp2.index) > 0:
            temp_mult = pd.to_numeric(temp2[1]) / pd.to_numeric(temp2[0])
            multiplier2 = temp_mult.sort_index(ascending=False).cumprod().sort_index(ascending=True)
        else:
            multiplier2 = pd.Series(1, index=multiplier1.index)
        multiplier = pd.concat([multiplier1, multiplier2], axis=1).fillna(method='bfill').fillna(1)
        multiplier[1] = multiplier[1].shift(-1).fillna(1)
        temp['Close'] = temp['Close'] * multiplier[0] * multiplier[1]
        temp['Open'] = temp['Open'] * multiplier[0] * multiplier[1]
        temp['High'] = temp['High'] * multiplier[0] * multiplier[1]
        temp['Low'] = temp['Low'] * multiplier[0] * multiplier[1]
        temp['Volume'] = temp['Volume'] / multiplier[1]

        del temp['Dividends']
        temp.to_csv(fileName)



from backtester.features.feature import Feature
from backtester.financial_fn import ma
import numpy as np


class ArgMaxFeature(Feature):

    @classmethod
    def computeForLookbackData(cls, featureParams, featureKey, currentFeatures, lookbackDataDf):
        data = lookbackDataDf[featureParams['featureName']]
        # avg = ma(data, featureParams['period'])
        if len(data) < 1:
            return 0
        return data[-featureParams['period']:].idxmax()



from backtester.features.feature import Feature


class VarianceFeature(Feature):
    # Computing for Instrument. By default defers to computeForLookbackData
    # MAKE SURE PNL is calculated BEFORE this feature
    @classmethod
    def computeForInstrument(cls, featureParams, featureKey, currentFeatures, instrument, instrumentManager):
        lookbackDataDf = instrument.getDataDf()
        pnlKey = 'pnl'
        countKey = 'count'
        if len(lookbackDataDf) <= 1 or instrumentManager is None:
            return 0
        lookbackMarketDataDf = instrumentManager.getDataDf()
        if 'pnlKey' in featureParams:
            pnlKey = featureParams['pnlKey']
        if 'countKey' in featureParams:
            countKey = featureParams['countKey']
        if len(lookbackMarketDataDf) <= 1:
            # first iteration
            return 0
        prevCount = lookbackMarketDataDf[countKey].iloc[-1]

        pnlDict = lookbackDataDf[pnlKey]
        varDict = lookbackDataDf[featureKey]
        if len(varDict) <= 1:
            return 0

        sqSum = 0 if (len(varDict) <= 1) else float(prevCount) * varDict.iloc[-2]

        prevAvgPnl = pnlDict.iloc[-2] / float(prevCount)
        newAvgPnl = pnlDict.iloc[-1] / float(prevCount + 1)
        newSqSum = sqSum + prevCount * (prevAvgPnl**2 - newAvgPnl**2) \
            + (pnlDict.iloc[-2] - pnlDict.iloc[-1] - newAvgPnl)**2

        return newSqSum / float(prevCount + 1)

    '''
    Computing for Market. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForMarket(cls, featureParams, featureKey, currentMarketFeatures, instrumentManager):
        pnlKey = 'pnl'
        countKey = 'count'
        lookbackMarketDataDf = instrumentManager.getDataDf()
        if len(lookbackMarketDataDf) <= 2 or instrumentManager is None:
            # First Iteration
            return 0
        if 'pnlKey' in featureParams:
            pnlKey = featureParams['pnlKey']
        if 'countKey' in featureParams:
            countKey = featureParams['countKey']

        prevCount = lookbackMarketDataDf[countKey].iloc[-2]

        pnlDict = lookbackMarketDataDf[pnlKey]
        varDict = lookbackMarketDataDf[featureKey]
        if len(varDict) <= 1:
            return 0

        sqSum = 0 if (len(varDict) <= 1) else float(prevCount) * varDict.iloc[-2]

        prevAvgPnl = pnlDict.iloc[-2] / float(prevCount)
        newAvgPnl = pnlDict.iloc[-1] / float(prevCount + 1)
        newSqSum = sqSum + prevCount * (prevAvgPnl**2 - newAvgPnl**2)\
            + (pnlDict.iloc[-2] - pnlDict.iloc[-1] - newAvgPnl)**2

        return newSqSum / float(prevCount + 1)



from backtester.features.feature import Feature
from backtester.financial_fn import msdev


class MovingSDevFeature(Feature):

    @classmethod
    def computeForLookbackData(cls, featureParams, featureKey, currentFeatures, lookbackDataDf):
        data = lookbackDataDf[featureParams['featureName']]
        sdev = data[-featureParams['period']:].std()
        if len(data) < 1:
            return 0
        return sdev



from backtester.features.feature import Feature
from backtester.financial_fn import ma
import numpy as np


class ScaleFeature(Feature):

    @classmethod
    def computeForLookbackData(cls, featureParams, featureKey, currentFeatures, lookbackDataDf):
    	data = lookbackDataDf[featureParams['featureName']][-featureParams['period']:]
    	return data.mul(featureParams['scale']).div(np.abs(data).sum())



from backtester.features.feature import Feature
from backtester.financial_fn import ma
import numpy as np


class Candle3Feature(Feature):

	#This checks the length of candle realbody

    @classmethod
    def computeForLookbackData(cls, featureParams, featureKey, currentFeatures, lookbackDataDf):
        if 'featureName' in featureParams and 'period' in featureParams:
        	data = lookbackDataDf[featureParams['featureName']]
        	o = data[-featureParams['period']]
        	c = data[-1]
        else:
        	o = lookbackDataDf['open'].iloc[-1]
        	c = lookbackDataDf['close'].iloc[-1]
        return np.abs(c-o)/float(c)



from backtester.executionSystem.base_execution_system import BaseExecutionSystem, InstrumentExection
from backtester.logger import *
import numpy as np


class SimpleExecutionSystem(BaseExecutionSystem):
    def __init__(self, enter_threshold=0.7, exit_threshold=0.55, longLimit=10,
                 shortLimit=10, capitalUsageLimit=0, lotSize=1, limitType='L', price=''):
        self.enter_threshold = enter_threshold
        self.exit_threshold = exit_threshold
        self.longLimit = longLimit
        self.shortLimit = shortLimit
        self.capitalUsageLimit = capitalUsageLimit
        self.lotSize = lotSize
        self.limitType = limitType
        self.priceFeature = price

    def getLongLimit(self, instrument):
        instrumentId = instrument.getInstrumentId()
        if isinstance(self.longLimit, dict):
            return self.convertLimit(self.longLimit[instrumentId], instrument)
        else:
            return self.convertLimit(self.longLimit, instrument)

    def getShortLimit(self, instrument):
        instrumentId = instrument.getInstrumentId()
        if isinstance(self.shortLimit, dict):
            return self.convertLimit(self.shortLimit[instrumentId], instrument)
        else:
            return self.convertLimit(self.shortLimit, instrument)

    def getLotSize(self, instrument):
        instrumentId = instrument.getInstrumentId()
        if isinstance(self.lotSize, dict):
            return self.convertLimit(self.lotSize[instrumentId], instrument)
        else:
            return self.convertLimit(self.lotSize, instrument)

    def convertLimit(self, value, instrument):
        if self.limitType == 'L':
            return value
        try:
            price = instrument.getDataDf()[self.priceFeature].iloc[-1]
            return np.floor(np.float(value) / price)
        except KeyError:
            logError(
                'You have specified Dollar Limit but Price Feature Key does not exist')

    def getExecutions(self, time, instrumentsManager, capital):
        # TODO:
        marketFeaturesDf = instrumentsManager.getDataDf()
        currentMarketFeatures = marketFeaturesDf.iloc[-1]
        currentPredictions = marketFeaturesDf['prediction'].iloc[-1]
        logInfo(str(currentPredictions))
        executions = []
        executions += self.exitPosition(time, instrumentsManager, currentPredictions)
        executions += self.enterPosition(time, instrumentsManager,
                                         currentPredictions, capital)
        return executions

    def exitPosition(self, time, instrumentsManager, currentPredictions, closeAllPositions=False):
        executions = []
        instruments = instrumentsManager.getAllInstrumentsByInstrumentId().values()
        for instrument in instruments:
            position = instrument.getCurrentPosition()
            if position == 0:
                continue
            if closeAllPositions:
                instrumentExec = InstrumentExection(
                    time, instrument.getInstrumentId(), np.abs(position), -np.sign(position))
                executions.append(instrumentExec)
            # take Profits
            elif self.exitCondition(instrumentsManager, instrument, currentPredictions):
                instrumentExec = InstrumentExection(
                    time, instrument.getInstrumentId(), np.abs(position), -np.sign(position))
                executions.append(instrumentExec)
                logInfo('EXIT TRADE: %s Size: %.2f B/S: %i ' % (
                        instrument.getInstrumentId(), np.abs(position), -np.sign(position)))

            # hack
            elif self.hackCondition(instrumentsManager):
                instrumentExec = InstrumentExection(
                    time, instrument.getInstrumentId(), np.abs(position), -np.sign(position))
                executions.append(instrumentExec)
                logInfo('HACK TRADE: %s Size: %.2f B/S: %i ' % (
                        instrument.getInstrumentId(), np.abs(position), -np.sign(position)))
        return executions

    def enterPosition(self, time, instrumentsManager, currentPredictions, capital):
        executions = []
        for instrumentId in currentPredictions.keys():
            instrument = instrumentsManager.getInstrument(instrumentId)
            if instrument is None:
                continue
            # Dont add if already at limit
            if self.atPositionLimit(capital, instrumentsManager, instrument):
                continue
            # Enter position if condition met
            if self.enterCondition(instrumentsManager, instrument, currentPredictions):
                buySell = self.getBuySell(instrument, currentPredictions)
                logInfo('ENTER TRADE: %s Size: %.2f B/S: %i ' % (
                        instrumentId, self.getLotSize(instrument), buySell))
                instrumentExec = InstrumentExection(time, instrumentId,
                                                    self.getLotSize(instrument), buySell)
                executions.append(instrumentExec)
        return executions

    def getBuySell(self, instrument, currentPredictions):
        instrumentId = instrument.getInstrumentId()
        return np.sign(currentPredictions[instrumentId] - 0.5)

    def enterCondition(self, instrumentsManager, instrument, currentPredictions):
        instrumentId = instrument.getInstrumentId()
        probBuy = currentPredictions[instrumentId]
        return np.abs(probBuy - 0.5) > (self.enter_threshold - 0.5)

    def atPositionLimit(self, capital, instrumentsManager, instrument):
        position = instrument.getCurrentPosition()
        instrumentId = instrument.getInstrumentId()
        if (position > self.getLongLimit(instrument)) or (position < -self.getShortLimit(instrument)):
            logWarn('At Position Limit for %s' % instrumentId)
            return True
        if capital < self.capitalUsageLimit:
            logWarn('Not Enough Capital')
            return True
        return False

    def exitCondition(self, instrumentsManager, instrument, currentPredictions):
        instrumentId = instrument.getInstrumentId()
        probBuy = currentPredictions[instrumentId]
        return np.abs(probBuy - 0.5) < (self.exit_threshold - 0.5)

    def hackCondition(self, instrumentsManager):
        return False



from backtester.features.feature import Feature


class MaxCapitalUsageFeature(Feature):

    '''
    Computing for Instrument. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForInstrument(cls, featureParams, featureKey, currentFeatures, instrument, instrumentManager):

        raise NotImplementedError
        return None

    '''
    Computing for Market. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForMarket(cls, featureParams, featureKey, currentMarketFeatures, instrumentManager):
        maxUsage = 0
        capitalUsageDict = instrumentManager.getDataDf()[featureKey]
        capitalKey = 'capital'
        if 'capitalKey' in featureParams:
            capitalKey = featureParams['capitalKey']
        if len(capitalUsageDict) <= 1:
            return 0
        capitalDict = instrumentManager.getDataDf()[capitalKey]
        capital = capitalDict.values[-2]
        capitalUsed = featureParams['initial_capital'] - capital
        maxUsage = capitalUsed if capitalUsed > capitalUsageDict.values[-2] else capitalUsageDict.values[-2]
        return maxUsage



from backtester.instruments.instrument import Instrument
from backtester.constants import *


class StockInstrument(Instrument):

    def __init__(self, stockInstrumentId, bookDataFeatures, tsParams):
        super(StockInstrument, self).__init__(stockInstrumentId, bookDataFeatures, tsParams)
        self.__stockInstrumentId = stockInstrumentId

    def getInstrumentType(self):
        return INSTRUMENT_TYPE_STOCK



from backtester.instrumentUpdates.instrument_update import InstrumentUpdate
from backtester.constants import *


class StockInstrumentUpdate(InstrumentUpdate):
    '''
    stockInstrumentId: symbol for stock
    timeOfUpdate: datetime object for time of update
    bookData: dictionary. keys which we support right now:
             bidPrice
             bidVolume
             askPrice
             askVolume
             volume
             open
             close
             high
             low
    '''
    def __init__(self, stockInstrumentId, tradeSymbol, timeOfUpdate, bookData):
        super(StockInstrumentUpdate, self).__init__(stockInstrumentId, tradeSymbol, timeOfUpdate, bookData)
        self.__stockInstrumentId = stockInstrumentId

    def getStockInstrumentId(self):
        return self.__stockInstrumentId

    def getTypeOfInstrument(self):
        return INSTRUMENT_TYPE_STOCK



from backtester.features.feature import Feature
import numpy as np

class PortfolioValueFeature(Feature):

    '''
    Computing for Instrument. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForInstrument(cls, featureParams, featureKey, currentFeatures, instrument, instrumentManager):      
        raise NotImplementedError
        return None


    '''
    Computing for Market. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForMarket(cls, featureParams, featureKey, currentMarketFeatures, instrumentManager):
        return featureParams['initial_capital'] + currentMarketFeatures[featureParams['pnl']]



from backtester.features.feature import Feature


class MomentumFeature(Feature):

    @classmethod
    def computeForLookbackData(cls, featureParams, featureKey, currentFeatures, lookbackDataDf):
        data = lookbackDataDf[featureParams['featureName']]
        if len(data.index) > featureParams['period']:
            m = (data[-1]/data[-featureParams['period']] -1 )* 100
        else:
            m = 0
        return m



from backtester.dataSource import *
from backtester.executionSystem import *
from backtester.features import *
from backtester.instruments import *
from backtester.instrumentUpdates import *
from backtester.metrics import *
from backtester.orderPlacer import *
from backtester.constants import *
from backtester.financial_fn import *
from backtester.instruments_manager import *
from backtester.logger import *
from backtester.lookback_data import *
from backtester.plotter import *
from backtester.state_writer import *
from backtester.trading_system_parameters import *
from backtester.trading_system import *
from backtester.version import *
from backtester.process_result import *



class Feature(object):

    '''
    Computing for Lookback data. Generic for instrument and market.
    '''
    @classmethod
    def computeForLookbackData(cls, featureParams, featureKey, currentFeatures, lookbackDataDf):
        raise NotImplementedError
        return None

    '''
    Computing for Instrument. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForInstrument(cls, featureParams, featureKey, currentFeatures, instrument, instrumentManager):
        lookbackDataDf = instrument.getDataDf()
        return cls.computeForLookbackData(featureParams, featureKey, currentFeatures, lookbackDataDf)

    '''
    Computing for Market. By default defers to computeForLookbackData
    '''
    @classmethod
    def computeForMarket(cls, featureParams, featureKey, currentMarketFeatures, instrumentManager):
        lookbackDataDf = instrumentManager.getDataDf()
        return cls.computeForLookbackData(featureParams, featureKey, currentMarketFeatures, lookbackDataDf)



from backtester.dataSource.data_source import DataSource
from backtester.instrumentUpdates import *
import os
from datetime import datetime
from backtester.dataSource.data_source_utils import groupAndSortByTimeUpdates
import csv
from backtester.logger import *
try:
    from urllib2 import urlopen
except ImportError:
    from urllib.request import urlopen


def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False


class QuantQuestDataSource(DataSource):
    def __init__(self, cachedFolderName, dataSetId, instrumentIds):
        self.__cachedFolderName = cachedFolderName
        self.__dataSetId = dataSetId
        self.ensureDirectoryExists(cachedFolderName, dataSetId)
        self.ensureAllInstrumentsFile(dataSetId)
        if instrumentIds is not None and len(instrumentIds) > 0:
            self.__instrumentIds = instrumentIds
        else:
            self.__instrumentIds = self.getAllInstrumentIds()
        self.__groupedInstrumentUpdates = self.getGroupedInstrumentUpdates()

    def ensureDirectoryExists(self, cachedFolderName, dataSetId):
        if not os.path.exists(cachedFolderName):
            os.mkdir(cachedFolderName, 0o755)
        if not os.path.exists(cachedFolderName + '/' + dataSetId):
            os.mkdir(cachedFolderName + '/' + dataSetId)

    def getFileName(self, instrumentId):
        return self.__cachedFolderName + self.__dataSetId + '/' + instrumentId + '.csv'

    def ensureAllInstrumentsFile(self, dataSetId):
        stockListFileName = self.__cachedFolderName + self.__dataSetId + '/' + 'stock_list.txt'
        if os.path.isfile(stockListFileName):
            return True
        url = 'https://raw.githubusercontent.com/Auquan/auquan-historical-data/master/qq2Data/%s/stock_list.txt' % (
            dataSetId)
        print(url)
        response = urlopen(url)
        status = response.getcode()
        if status == 200:
            print('Downloading list of stocks to file: %s' % (stockListFileName))
            with open(stockListFileName, 'w') as f:
                f.write(response.read().decode('utf8'))
            return True
        else:
            logError('File not found. Please check internet')
            return False

    def getAllInstrumentIds(self):
        stockListFileName = self.__cachedFolderName + self.__dataSetId + '/' + 'stock_list.txt'
        if not os.path.isfile(stockListFileName):
            logError('Stock list file not present. Please try running again.')
            return []

        with open(stockListFileName) as f:
            content = f.readlines()
        # you may also want to remove whitespace characters like `\n` at the end of each line
        content = [x.strip() for x in content]
        return content

    def downloadFile(self, dataSetId, instrumentId, downloadLocation):
        url = 'https://raw.githubusercontent.com/Auquan/auquan-historical-data/master/qq2Data/%s/%s.csv' % (
            dataSetId, instrumentId)
        response = urlopen(url)
        status = response.getcode()
        if status == 200:
            print('Downloading %s data to file: %s' % (instrumentId, downloadLocation))
            with open(downloadLocation, 'w') as f:
                f.write(response.read().decode('utf8'))
            return True
        else:
            logError('File not found. Please check settings!')
            return False

    def getInstrumentUpdateFromRow(self, instrumentId, row):
        bookData = row
        for key in bookData:
            if is_number(bookData[key]):
                bookData[key] = float(bookData[key])
        timeKey = ''
        timeOfUpdate = datetime.strptime(row[timeKey], '%Y-%m-%d %H:%M:%S')
        bookData.pop(timeKey, None)
        inst = StockInstrumentUpdate(stockInstrumentId=instrumentId,
                                     tradeSymbol=instrumentId,
                                     timeOfUpdate=timeOfUpdate,
                                     bookData=bookData)
        return inst

    def getGroupedInstrumentUpdates(self):
        allInstrumentUpdates = []
        for instrumentId in self.__instrumentIds:
            print('Processing data for stock: %s' % (instrumentId))
            fileName = self.getFileName(instrumentId)
            if not os.path.exists(self.__cachedFolderName):
                os.mkdir(self.cachedFolderName, 0o755)
            if not os.path.isfile(fileName):
                if not self.downloadFile(self.__dataSetId, instrumentId, fileName):
                    logError('Skipping %s:' % (instrumentId))
                    continue
            with open(self.getFileName(instrumentId)) as f:
                records = csv.DictReader(f)
                for row in records:
                    inst = self.getInstrumentUpdateFromRow(instrumentId, row)
                    allInstrumentUpdates.append(inst)

        groupedInstrumentUpdates = groupAndSortByTimeUpdates(allInstrumentUpdates)
        return groupedInstrumentUpdates

    def emitInstrumentUpdates(self):
        for timeOfUpdate, instrumentUpdates in self.__groupedInstrumentUpdates:
            yield([timeOfUpdate, instrumentUpdates])
